/*
 *  ActiveSegmentation.java
 *
 *
 * Copyright (c) 2006, LRI; Univ. Paris-Sud, CNRS, France.
 *
 * This file is free software, licenced under the GNU Library General Public
 * License, Version 2, June 1991.
 *
 * A copy of this licence is included in the distribution in the file
 * licence.html, and is also available at
 * http://gate.ac.uk/gate/licence.html.
 *
 * Based on SentenceSplitter.java,v 1.29 2005/06/30 17:03:08 valyt Exp
 *
 * Written by Thomas Heitz, LRI; Univ. Paris-Sud, CNRS, France.
 *
 *  heitz, 13/3/2006
 *
 *  $Id: ActiveSegmentation.jav,v 1.2 2001/11/14 17:15:43 oana Exp $
 *
 * TODO:
 *
 * o progress status bars aren't displayed
 *
 * o read-only document to annotate (setEditable in future version of gate)
 *
 * o add a feature to Abbr and CapWord that is the number of characters
 * between AbbrCertain|AbbrPossible and AbbrCertain|AbbrPossible,
 * CapWordPossible|CapWordPossible and CapWordPossible|CapWordPossible.
 */

package segmentation;

import gate.*;
import gate.creole.*;
import gate.creole.gazetteer.DefaultGazetteer;
import gate.event.ProgressListener;
import gate.event.StatusListener;
import gate.util.GateRuntimeException;
import gate.util.InvalidOffsetException;
import gate.util.LuckyException;
import gate.util.GateException;
import gate.creole.ExecutionException;
import gate.creole.ml.*;
import gate.gui.*;
import gate.annotation.*;
import java.io.*;
import java.net.*;
import java.beans.*;
import javax.swing.*;
import java.util.regex.*;
import java.util.*;
import weka.core.Instances.*;
import weka.classifiers.Classifier;
import weka.classifiers.evaluation.EvaluationUtils;

/**
 * A sentence splitter that use an active learning procedure.
 *
 * Its purpose is to segment a corpus of texts into sentences but not on
 * abbreviations.
 *
 * You will have to tag a sample (Parameter: corpusSampleDocument) of the
 * full corpus (Parameter: corpus), steps 2, 3, 4, 5 and 6.
 *
 * At the end of the process, step 7, you will have all files in the corpus
 * (Parameter: corpus) rewritten in new files with one sentence per line and
 * two lists of abbreviations and capitalized words. All files will be put
 * in the results directory (Parameter: resultsDirectoryURL).
 *
 * 1. Compute statistics on the full corpus
 * 2. Use lists and statistics to do a first tagging of abbreviations
 * 3. Learn and show the learning error on abbreviations
 * 4. First tagging of capitalized words
 * 5. Learn and show the learning error on capitalized words
 * 6. Determine Cap. words according to Abbreviations and conversely
 * 7. Apply learned model on full corpus to segment sentences
 */
public class ActiveSegmentation extends AbstractLanguageAnalyser {

  public static final
      String SPLIT_SAMPLE_DOCUMENT_PARAMETER_NAME = "corpusSampleDocument";
  public static final
      String SPLIT_CORPUS_PARAMETER_NAME = "corpus";
  public static final
      String SPLIT_LINE_ANNOTATIONS_TO_SKIP = "lineAnnotationsToSkipList";
  public static final
      String SPLIT_INPUT_AS_PARAMETER_NAME = "inputASName";
  public static final
      String SPLIT_OUTPUT_AS_PARAMETER_NAME = "outputASName";
  public static final
      String SPLIT_ENCODING_PARAMETER_NAME = "encoding";
  public static final
      String SPLIT_RESOURCES_URL_PARAMETER_NAME = "resourcesDirectoryURL";
  public static final
      String SPLIT_GAZ_URL_PARAMETER_NAME = "gazetteerFileURL";
  public static final
      String SPLIT_TRANSD_URL_PARAMETER_NAME = "transducerDirectoryURL";
  public static final
      String SPLIT_PREFIXES_URL_PARAMETER_NAME = "prefixesFileURL";
  public static final
      String SPLIT_SUFFIXES_URL_PARAMETER_NAME = "suffixesFileURL";
  public static final
      String SPLIT_TAGS_URL_PARAMETER_NAME = "tagsFileURL";
  public static final
      String SPLIT_LEXICON_URL_PARAMETER_NAME = "lexiconFileURL";
  public static final
      String SPLIT_RESULTS_URL_PARAMETER_NAME = "resultsDirectoryURL";

  private gate.Document corpusSampleDocument;
  private gate.Corpus corpus;
  private List lineAnnotationsToSkipList;
  private String inputASName;
  private String outputASName;
  private String encoding;
  private URL resourcesDirectoryURL;
  private URL gazetteerFileURL;
  private URL transducerDirectoryURL;
  private URL prefixesFileURL;
  private URL suffixesFileURL;
  private URL tagsFileURL;
  private URL lexiconFileURL;
  private URL resultsDirectoryURL;
  private boolean POSTagging;

  private static final boolean DEBUG = false;
  private ProgressListener pListener;
  private StatusListener sListener;
  private DefaultGazetteer gazetteer;
  private Transducer transducer_clean;
  private Transducer transducer02_2;
  private Transducer transducer02_2_1;
  private Transducer transducer02_2_2;
  private Transducer transducer03_1;
  private Transducer transducer04_1;
  private Transducer transducer04_1_1;
  private Transducer transducer04_1_2;
  private Transducer transducer05_1;
  private Transducer transducer07_3;
  private SerialAnalyserController annieController;
  private MachineLearningPR abbml;
  private MachineLearningPR pnml;
  private String lineAnnotationsToSkip;
  private String lineAnnotationsToSkipRule;

  private gate.gui.MainFrame mainFrame;
  private DocumentEditor docEditor;
  private java.util.HashMap abbreviations;
  private java.util.HashMap capExpressions;
  private java.util.HashMap prefixes;
  private java.util.HashMap suffixes;
  private java.util.HashMap tags;
  private double[] statisticsValues = {5,3,20,5,3,5,2,2};
  private boolean automatic;
  private Integer idAbbrCertain;
  private Integer idAbbrPossible;
  private Integer idAbbrImpossible;
  private Integer idCapWordCertain;
  private Integer idCapWordPossible;
  private Integer idCapWordImpossible;

  // Initialize dialogs
  final JDialog dialog0 = new JDialog(mainFrame, "Start", false);
  final JDialog dialog1 = new JDialog(mainFrame,
                    "Step 1/7 - Compute statistics", false);
  final JDialog dialog2 = new JDialog(mainFrame,
                    "Step 2/7 - First tagging of abbreviations", false);
  final JDialog dialog3 = new JDialog(mainFrame,
                    "Step 3/7 - Learn abbreviations", false);
  final JDialog dialog4 = new JDialog(mainFrame,
                    "Step 4/7 - First tagging of capitalized words", false);
  final JDialog dialog5 = new JDialog(mainFrame,
                    "Step 5/7 - Learn capitalized words", false);
  final JDialog dialog6 = new JDialog(mainFrame,
                    "Step 6/7 - Classify possible capitalized words and "+
                    "abbreviations", false);
  final JDialog dialog7 = new JDialog(mainFrame,
                    "Step 7/7 - Apply on full corpus", false);

  /**
   * Initialize the resources.
   */
  public Resource init() throws ResourceInstantiationException {

      fireProgressChanged(0);
      try {
          mainFrame = Main.getMainFrame();

          // add a specific DocumentEditor
          (gate.Gate.getCreoleRegister())
              .addDirectory(resourcesDirectoryURL);

      } catch (Exception e) {
          throw new ResourceInstantiationException(e);
      }

      if (inputASName != null && inputASName.equals("")) {
          inputASName = null;
      }
      if (outputASName != null && outputASName.equals("")) {
          outputASName = null;
      }

      pListener = null;
      sListener = new StatusListener() {
              public void statusChanged (String text) {
                  fireStatusChanged(text);
              }
          };

      abbreviations = new java.util.HashMap();
      capExpressions = new java.util.HashMap();

      prefixes = new java.util.HashMap();
      suffixes = new java.util.HashMap();
      tags = new java.util.HashMap();

      lineAnnotationsToSkip =
          lineAnnotationsToSkipList.toString().replaceAll("[,\\[\\]]", "");
      lineAnnotationsToSkipRule =
          lineAnnotationsToSkip.trim().replaceAll(" +", "}|{");

      fireStatusChanged("Reading prefixes, affixes and tags files");
      try {
          BufferedReader br =  new BufferedReader(new FileReader(
                            new File(new URI(prefixesFileURL.toString()))));

          String lineContent = br.readLine();
          while (lineContent != null) {
              // read prefixes file and store them into a hashtable
              prefixes.put(lineContent.trim(), null);
              lineContent = br.readLine();
          }
          br.close();

          br =  new BufferedReader(new FileReader(
                            new File(new URI(suffixesFileURL.toString()))));

          lineContent = br.readLine();
          while (lineContent != null) {
              // read affixes file and store them into a hashtable
              suffixes.put(lineContent.trim(), null);
              lineContent = br.readLine();
          }
          br.close();

          br =  new BufferedReader(new FileReader(
                            new File(new URI(tagsFileURL.toString()))));

          lineContent = br.readLine();
          while (lineContent != null) {
              // read tags file and store them into a hashtable
              tags.put(lineContent.trim(), null);
              lineContent = br.readLine();
          }
          br.close();

      } catch (Exception e) {
          throw new ResourceInstantiationException(e);
      }

      fireStatusChanged("Creating corpus and pipeline");
      fireProgressChanged(50);
      // create a corpus and add it to this PR
      corpus = Factory.newCorpus("ActiveSegmentation_" + Gate.genSym());

      // create a document pipeline and add this PR to it, display the PR
      SerialController sc = (SerialController)Factory.createResource(
          "gate.creole.SerialController", Factory.newFeatureMap(),
          Factory.newFeatureMap(), "ActiveSegmentation_" + Gate.genSym());
      sc.add(this);
      mainFrame.select(sc);

      // if there is only one document loaded use it as corpusSampleDocument
      List list = Gate.getCreoleRegister().getPublicLrInstances();
      int numDocument = 0;
      for (int i = 0; i < list.size(); i++) {
          if (Gate.getCreoleRegister().getPublicLrInstances().get(i)
              instanceof gate.Document) {
              corpusSampleDocument = (gate.corpora.DocumentImpl)
                  Gate.getCreoleRegister().getPublicLrInstances().get(i);
              numDocument++;
          }
      }
      if (numDocument > 1) { corpusSampleDocument = null; }

      fireProgressChanged(100);
      fireProcessFinished();
      return this;
  } // public Resource init()

  /**
   * Reinitialize the resources.
   */
  public void reInit() throws ResourceInstantiationException {

      String fileName;
      String grammar;

      try {
      if (transducer_clean == null) {
      // create transducer
      fileName = "clean.jape";
      grammar =
"Phase: Clean\n"+
"Input: AbbrCertain AbbrPossible AbbrImpossible\n"+
"CapWordCertain CapWordPossible CapWordImpossible\n"+
"Abbreviation CapitalizedWord\n"+
"Options: control = appelt\n"+
"\n"+
"Rule: Clean\n"+
"({AbbrCertain}|{AbbrPossible}|{AbbrImpossible}\n"+
"|{CapWordCertain}|{CapWordPossible}|{CapWordImpossible}\n"+
"|{Abbreviation}|{CapitalizedWord}):word\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"annotations.removeAll(AS);\n"+
"}\n";
      transducer_clean = createTransducer(grammar, document, inputASName,
                                          inputASName, fileName, encoding);
      } else {
          transducer_clean.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      fireProgressChanged(0);
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer_clean.addProgressListener(pListener);
      transducer_clean.addStatusListener(sListener);
      transducer_clean.execute();
      transducer_clean.removeProgressListener(pListener);
      transducer_clean.removeStatusListener(sListener);
      fireProcessFinished();

      } catch (ExecutionException e) {
          throw new ResourceInstantiationException(e);
      }

      init();
  }

  /**
   * Execute step by step the segmentation.
   */
  public void execute() throws ExecutionException {

      interrupted = false;
      automatic = false;

      step00();

      // Step 00
      Object[] possibleValues0 = {"Start","Cancel"};
      final JOptionPane optionPane0 = new JOptionPane(
     "Welcome in the Active Segmentation module.\nIts purpose is to "+
     "segment a corpus of texts into sentences but not on abbreviations.\n"+
     "\nBe careful, the files used to compute statistics and to\n"+
     "be annotated, should consist of one paragraph per\n"+
     "line and have no space at the beginning and end of line.\n\n"+
     "You will have to tag a sample (Parameter: corpusSampleDocument) of\n"+
     "the full corpus (Parameter: corpus), steps 2, 3, 4, 5 and 6.\n\n"+
     "At the end of the process, step 7, you will have all files in the\n"+
     "corpus (Parameter: corpus) rewritten in new files with one\n"+
     "sentence per line and two lists of abbreviations and capitalized "+
     "words.\n"+
     "All files will be put in the results directory "+
     "(Parameter: resultsDirectoryURL).\n\n"+
     "Note that Perl scripts are provided in the tools directory to "+
     "prepare data.\n"+
     "See other parameters to adapt the program to your needs.\n\n"+
     "Here are the steps you will have to perform:\n\n"+
     "1. Compute statistics on the full corpus\n"+
     "2. Use lists and statistics to do a first tagging of abbreviations\n"+
     "3. Learn and show the learning error on abbreviations\n"+
     "4. First tagging of capitalized words\n"+
     "5. Learn and show the learning error on capitalized words\n"+
     "6. Determine Cap. words according to Abbreviations and conversely\n"+
     "7. Apply learned model on full corpus to segment sentences\n\n",
            JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
            possibleValues0, "Start");
      dialog0.setContentPane(optionPane0);
      optionPane0.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane0.getValue();

                  if (dialog0.isVisible()
                      && (e.getSource() == optionPane0)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      dialog0.setVisible(false);
                      if (inputValue.equals("Start")) {
                          dialog1.setVisible(true);
                      } else if (inputValue.equals("Cancel")) {
                          dialog0.setVisible(false);
                          dialog0.dispose();
                      }
                  }
              }
          });
      dialog0.pack();
      dialog0.setLocation(mainFrame.getWidth()/2-dialog0.getWidth()/2, 0);
      dialog0.setVisible(true);
      optionPane0.selectInitialValue();

      // Step 01
      Object[] possibleValues1 = {"Compute statistics"};
      final JOptionPane optionPane1 = new JOptionPane(
            "Compute statistics on the full corpus\n"+
            "(Parameter: corpus). It will take one minute\n"+
            "per 10 Megabytes on a 3GHz/512MB PC.\n"+
            "It's an unattended process.\n",
            JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
            possibleValues1, "Compute statistics");
      dialog1.setContentPane(optionPane1);
      optionPane1.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane1.getValue();
      
                  if (dialog1.isVisible()
                      && (e.getSource() == optionPane1)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Compute statistics")
                          && corpus.isEmpty()) {

                          mainFrame.select(corpus);
                          JOptionPane.showMessageDialog(mainFrame,
                              "Please add at least one file in the corpus",
                              "Error", JOptionPane.ERROR_MESSAGE);
                          optionPane1
                              .setValue(JOptionPane.UNINITIALIZED_VALUE);
                          dialog1.toFront();

                      } else if (inputValue.equals("Compute statistics")) {
                          mainFrame.select(document);
                          step01();
                          dialog1.setVisible(false);
                          dialog2.setVisible(true);
                      }
                  }
              }
          });
      dialog1.pack();
      dialog1.setLocation(mainFrame.getWidth()/2-dialog1.getWidth()/2, 0);

      // Step 02
      Object[] possibleValues2 = {"Do it"};
      final JOptionPane optionPane2 = new JOptionPane(
           "Use Annie Tokeniser, Gazetteer and optionally POS\n"+
           "Tagger. Use lists and previous statistics to do a first\n"+
           "tagging of abbreviations in the sample file\n"+
           "(Parameter: corpusSampleDocument) of the full corpus.\n\n"+
           "You can add Jape rules files named user_abbr_1.jape and\n"+
           "user_abbr_2.jape in the transducer directory\n"+
           "(Parameter: transducerDirectoryURL) to add your own rules.\n\n"+
           "Note: you can use script in tools directory with command\n"+
           "'perl random-selection-in-file.pl -p 5 -in fullCorpus\n"+
           "-out sampleCorpus' to randomly select lines in a fullCorpus\n"+
           "and obtain a sampleCorpus file downsized to 5%.\n",
           JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
           possibleValues2, "Do it");
      dialog2.setContentPane(optionPane2);
      optionPane2.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane2.getValue();
      
                  if (dialog2.isVisible()
                      && (e.getSource() == optionPane2)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Do it")) {

                          document = corpusSampleDocument;
                          step02_1();
                          step02_2();

                          // expand annotations table
                          if (docEditor != null) {
                              docEditor.activeButton("Annotation Sets");
                              docEditor.expandAnnotationTable();
                          }

                          step02_4(5, 3, 20);

                          dialog2.setVisible(false);
                          int[] p = {5, 3, 20};
                          (new StatisticsDialog(mainFrame, false, p))
                              .setVisible(true);
                      }
                  }
              }
          });
      dialog2.pack();
      dialog2.setLocation(mainFrame.getWidth()/2-dialog2.getWidth()/2, 0);

      // Step 03
      Object[] possibleValues3 = {"Learn"};
      final JOptionPane optionPane3 = new JOptionPane(
          "Learn abbreviations and show the learning error.\n\n"+
          "You can add numeric features named user_abbr_1 and\n"+
          "user_abbr_2 to Abbreviations annotations to be used in\n"+
          "this learning. Don't forget to tick the right checkbox too.\n",
            JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
            possibleValues3, "Learn");
      dialog3.setContentPane(optionPane3);
      optionPane3.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane3.getValue();
      
                  if (dialog3.isVisible()
                      && (e.getSource() == optionPane3)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Learn")) {
                          step03_1();
                          dialog3.setVisible(false);
                       int[]a={1,1,1,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0};
                          (new LearnAbbreviationsDialog(mainFrame, false,
                                                        step03_2(a), a))
                              .setVisible(true);
                      }
                  }
              }
          });
      dialog3.pack();
      dialog3.setLocation(mainFrame.getWidth()/2-dialog3.getWidth()/2, 0);

      // Step 04
      Object[] possibleValues4 = {"Do it"};
      final JOptionPane optionPane4 = new JOptionPane(
           "Use Annie Tokeniser, Gazetteer, Sentence Splitter and POS\n"+
           "Tagger. Use lists and previous statistics to do a first\n"+
           "tagging of capitalized words in the sample file\n"+
           "(Parameter: corpusSampleDocument) of the full corpus.\n\n"+
           "You can add Jape rules files named user_cap_1.jape and\n"+
           "user_cap_2.jape in the transducer directory\n"+
           "(Parameter: transducerDirectoryURL) to add your own rules.\n",
            JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
            possibleValues4, "Do it");
      dialog4.setContentPane(optionPane4);
      optionPane4.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane4.getValue();
      
                  if (dialog4.isVisible()
                      && (e.getSource() == optionPane4)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Do it")) {
                          step04_1();
                          step04_2(5, 3, 5);
                          dialog4.setVisible(false);
                          int[] p = {5, 3, 5};
                          (new Statistics2Dialog(mainFrame, false, p))
                              .setVisible(true);
                      }
                  }
              }
          });
      dialog4.pack();
      dialog4.setLocation(mainFrame.getWidth()/2-dialog4.getWidth()/2, 0);

      // Step 05
      Object[] possibleValues5 = {"Learn"};
      final JOptionPane optionPane5 = new JOptionPane(
          "Learn capitalized words and show the learning error.\n\n"+
          "You can add numeric features named user_cap_1 and\n"+
          "user_cap_2 to CapitalizedWord annotations to be used in\n"+
          "this learning. Don't forget to tick the right checkbox too.\n",
            JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
            possibleValues5, "Learn");
      dialog5.setContentPane(optionPane5);
      optionPane5.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane5.getValue();
      
                  if (dialog5.isVisible()
                      && (e.getSource() == optionPane5)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Learn")) {
                          step05_1();
                          dialog5.setVisible(false);
                       int[]a={1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0};
                          (new LearnCapitalizedWordsDialog(mainFrame, false,
                                                        step05_2(a), a))
                              .setVisible(true);
                      }
                  }
              }
          });
      dialog5.pack();
      dialog5.setLocation(mainFrame.getWidth()/2-dialog5.getWidth()/2, 0);

      // Step 06
      Object[] possibleValues6 = {"Do it"};
      final JOptionPane optionPane6 = new JOptionPane(
           "Determine capitalized words according to abbreviations and "+
           "conversely.\n",
           JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
           possibleValues6, "Do it");
      dialog6.setContentPane(optionPane6);
      optionPane6.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane6.getValue();

                  if (dialog6.isVisible()
                      && (e.getSource() == optionPane6)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Do it")) {
                          dialog6.setVisible(false);
                          int[] p = {2, 2};
                          (new Step06Dialog(mainFrame, false, p))
                              .setVisible(true);
                      }
                  }
              }
          });
      dialog6.pack();
      dialog6.setLocation(mainFrame.getWidth()/2-dialog6.getWidth()/2, 0);

      // Step 07
      Object[] possibleValues7 = {"Apply on full corpus"};
      final JOptionPane optionPane7 = new JOptionPane(
            "Apply learned model on all files in the full corpus\n"+
            "(Parameter: corpus). It will take eight\n"+
            "hours per 10 Megabytes on a 3GHz/512MB PC.\n"+
            "It's an unattended process. Resulting files will be put in\n"+
            "results directory (Parameter: resultsDirectoryURL).\n\n"+
            "Note: you can use script in tools directory with command\n"+
            "'perl split.pl -b 300k fullCorpus splittedCorpus' to split\n"+
            "fullCorpus file into several splittedCorpus files of 300Kb.\n"+
            "Adapt the size of split files to the power of your PC.\n",
            JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,
            possibleValues7, "Apply on full corpus");
      dialog7.setContentPane(optionPane7);
      optionPane7.addPropertyChangeListener(
          new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  String prop = e.getPropertyName();
                  String inputValue = (String)optionPane7.getValue();

                  if (dialog7.isVisible()
                      && (e.getSource() == optionPane7)
                      && prop.equals(JOptionPane.VALUE_PROPERTY)) {

                      if (inputValue.equals("Apply on full corpus")) {

                          // if the same document is used in the corpus too
                          // don't delete it
                          if (!corpus.contains(document)) {
                              Factory.deleteResource(document);
                          }

                          automatic = true;
                          docEditor = null;
                          // erase lists of words previously generated
                          try {
                          (new File(new URI((new URL(resultsDirectoryURL
                             +"/abbreviations.txt")).toString()))).delete();
                          (new File(new URI((new URL(resultsDirectoryURL
                             +"/abbreviations_possible.txt"))
                                            .toString()))).delete();
                          (new File(new URI((new URL(resultsDirectoryURL
                         +"/capitalized_words.txt")).toString()))).delete();
                          (new File(new URI((new URL(resultsDirectoryURL
                         +"/capitalized_words_possible.txt")).toString())))
                              .delete();
                          } catch (Exception ex) {
                              ex.printStackTrace();
                              return;
                          }

                          // for each document in the corpus
                          while (!corpus.isEmpty()) {

                              document = (Document)corpus.get(0);

                              step02_1();
                              step02_2();
                              step02_4(statisticsValues[0],
                                       statisticsValues[1],
                                       statisticsValues[2]);

                              step03_1();

                              step04_1();
                              step04_2(statisticsValues[3],
                                       statisticsValues[4],
                                       statisticsValues[5]);

                              step05_1();

                              step07_1();
                              step07_2();
                              step07_3();

                              // unload the document to free memory
                              if (transducer_clean != null) {
                                  transducer_clean.setDocument(null); }
                              annieController.setCorpus(null);
                              gazetteer.setDocument(null);
                              transducer02_2.setDocument(null);
                              transducer03_1.setDocument(null);
                              abbml.setDocument(null);
                              transducer04_1.setDocument(null);
                              transducer05_1.setDocument(null);
                              pnml.setDocument(null);
                              transducer07_3.setDocument(null);
                              corpus.remove(0);
                              Factory.deleteResource(document);
                              document = null;
                          }
                          dialog7.setVisible(false);
                      }
                  }
              }
          });
      dialog7.pack();
      dialog7.setLocation(mainFrame.getWidth()/2-dialog7.getWidth()/2, 0);
 
 } // execute()

  /**
   * Step 00
   *
   * Display document and annotations set. Load ANNIE Document Reset,
   * English Tokeniser, Gazetteer, Sentence Splitter and POS Tagger.
   *
   */
  private void step00() {

      mainFrame.select(corpusSampleDocument);

      java.util.List vr = Gate.getCreoleRegister().getPublicVrInstances();

      // get DocumentEditor of the current document
      docEditor = null;
      for (int i = 0; i < vr.size(); i++) {
          if (vr.get(i) instanceof segmentation.DocumentEditor
          && (((DocumentEditor)vr.get(i)).getDocument()
              .equals(corpusSampleDocument))) {
              docEditor = (DocumentEditor)vr.get(i);
              break;
          }
      }

      try {
      if (annieController == null) {
      annieController = (SerialAnalyserController) Factory.createResource(
          "gate.creole.SerialAnalyserController", Factory.newFeatureMap(),
          Factory.newFeatureMap(), "ANNIE_" + Gate.genSym()
          );

      FeatureMap params = Factory.newFeatureMap();
      ProcessingResource pr = (ProcessingResource)Factory.createResource(
           "gate.creole.annotdelete.AnnotationDeletePR", params);
      annieController.add(pr);
      pr = (ProcessingResource)Factory.createResource(
           "gate.creole.tokeniser.DefaultTokeniser", params);
      annieController.add(pr);
      // POS tagging is optional
      if (POSTagging) {
          pr = (ProcessingResource)Factory.createResource(
               "gate.creole.gazetteer.DefaultGazetteer", params);
          annieController.add(pr);
          pr = (ProcessingResource)Factory.createResource(
               "gate.creole.splitter.SentenceSplitter", params);
          annieController.add(pr);
          pr = (ProcessingResource)Factory.createResource(
               "gate.creole.POSTagger", params);
          annieController.add(pr);
      }
      }

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 01.
   *
   * Compute statistics on possible abbreviation words and possible
   * capitalized words on the whole corpus.
   */
  private void step01() {

      String word, previous, next;
      Matcher matcherAbbreviation;
      Matcher matcherCapitalizedExpression;
      Matcher matcher3Tokens;
      java.util.HashMap previousWordsA;
      java.util.HashMap nextWordsA;
      java.util.HashMap previousWordsCE;
      java.util.HashMap nextWordsCE;
      java.util.HashMap lexicon;
      java.util.Iterator iterator;
      int start;
      PrintWriter out;
      BufferedReader br;
      URL path;
      String lineContent, tag;
      String[] array;

      previousWordsA = new java.util.HashMap();
      nextWordsA = new java.util.HashMap();
      previousWordsCE = new java.util.HashMap();
      nextWordsCE = new java.util.HashMap();
      lexicon = new java.util.HashMap();
      int[] a = {0,0,0,0,0,0};
      int[] a2 = {0,0,0,0,0,0,0};
//      pListener = new IntervalProgressListener(0, 100);
//      super.addProgressListener(pListener);
//      super.addStatusListener(sListener);

      // concatenate files in corpus in one file and delete all line
      // begining with a lineAnnotationsToSkip XML annotation
      fireStatusChanged("Concatenate files in corpus");
      fireProgressChanged(0);
      try {
      path = new URL(resultsDirectoryURL+"/corpusInOneDocument.txt");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                      new File(new URI(path.toString())))));
      // for each document in the corpus
      for (int i = 0; i < corpus.size(); i++) {

          document = (Document)corpus.get(i);
          br = new BufferedReader(new FileReader(new File(
                             new URI(document.getSourceUrl().toString()))));

          lineContent = br.readLine();
          while (lineContent != null) {
              array = lineContent.trim().split("[<>]");
              tag = (array.length > 1) ? array[1] : "";
              if (!lineAnnotationsToSkipList.contains(tag)) {
                  lineContent = lineContent.replaceAll("<[^>]+>", "");
                  out.println(lineContent);
              }
              lineContent = br.readLine();
          }
          br.close();
      }
      out.close();

      gate.corpora.DocumentImpl corpusInOneDocument =
          (gate.corpora.DocumentImpl)Factory.newDocument(path, encoding);
//      corpusInOneDocument.setMarkupAware(new Boolean(true));
      document = corpusInOneDocument;

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }

      fireStatusChanged("Compute statistics");
      fireProgressChanged(10);
      String l = "[^?!\"\'`(),;\\\\/\\p{Space}\\p{Cntrl}]";
      matcherAbbreviation = Pattern.compile(
          "("+l+"+)\\.[\\p{Space}?!\"\'),;\\\\/:.]")
          .matcher(document.getContent().toString());

      while (matcherAbbreviation.find()) {
          word = matcherAbbreviation.group(1);
          // get abbreviations from the text
          abbreviations.put(word, a.clone());
          previousWordsA.put(word, new java.util.HashMap());
          nextWordsA.put(word, new java.util.HashMap());
      }

      String separator =
       "(?:(?:[ \\t]+((?:(?:and)|&|(?:for)|(?:to)|(?:of))[ \\t]+)?)"+
          "|(?:\'[sS][ \\t]+))";
      String expression = "(\\p{Upper}"+l+"*)";
      matcherCapitalizedExpression = Pattern.compile(
          expression+"(?:"+separator+expression+")*")
          .matcher(document.getContent().toString());

      while (matcherCapitalizedExpression.find()) {
          word = matcherCapitalizedExpression.group();
          String[] sa = word.split("\\p{Space}");

          for (int i = 0; i < sa.length; i++) {
              if (!capExpressions.containsKey(sa[i])) {
                  capExpressions.put(sa[i], a2.clone());
              }
              // add the number of tokens in this expression to each of its
              // token feature
              ((int[])capExpressions.get(sa[i]))[5] += sa.length;
              previousWordsCE.put(sa[i], new java.util.HashMap());
              nextWordsCE.put(sa[i], new java.util.HashMap());
          }
      }

      fireProgressChanged(20);
      matcher3Tokens = Pattern.compile(
          "([?!\"\'`(),;:.\\\\/]|[\\n\\r]+|"+l+"+)"+
          "[ \\t]*("+l+"+)[ \\t]*"+
          "([?!\"\'`(),;:.\\\\/]|[\\n\\r]+|"+l+"+)")
          .matcher(document.getContent().toString());

      start = 0;
      while (matcher3Tokens.find(start)) {
          word = matcher3Tokens.group(2);
          previous = (matcher3Tokens.group(1).matches("[\\n\\r?!]+")) ?
              "new$$line." : matcher3Tokens.group(1);
          next = (matcher3Tokens.group(3).matches("[\\n\\r]+")) ?
              "New$$line" : matcher3Tokens.group(3);

          if (word.endsWith(".") && (word.length() > 1)) {
              word = word.substring(0, word.length()-1);

              if (abbreviations.containsKey(word)) {

                  if (next.substring(0,1).toUpperCase()
                      .equals(next.substring(0,1))) {
                      // occurrences of possible abbreviation words with
                      // dot in ambiguous position
                      ((int[])abbreviations.get(word))[1]++;

                  } else {
                      // occurrences of possible abbreviation words with
                      // dot in unambiguous position
                      ((int[])abbreviations.get(word))[0]++;
                  }

                  ((java.util.HashMap)previousWordsA.get(word))
                      .put(previous, null);
                  ((java.util.HashMap)nextWordsA.get(word))
                      .put(next, null);
              }

          } else {
              if (abbreviations.containsKey(word)) {
                  // occurrences of possible abbreviation words without dot
                  ((int[])abbreviations.get(word))[2]++;
              }
          }

          if (word.substring(0,1).toUpperCase()
              .equals(word.substring(0,1))) {

              if (capExpressions.containsKey(word)) {

                  if (previous.endsWith(".")) {
                      // occurrences of possible capitalized expressions
                      // capitalized in ambiguous position
                      ((int[])capExpressions.get(word))[1]++;

                  } else {
                      // occurrences of possible capitalized expressions
                      // capitalized in unambiguous position
                      ((int[])capExpressions.get(word))[0]++;
                  }

                  ((java.util.HashMap)previousWordsCE.get(word))
                      .put(previous, null);
                  ((java.util.HashMap)nextWordsCE.get(word))
                      .put(next, null);
              }

          } else {
              if (capExpressions.containsKey(word.toUpperCase())) {
                  // occurrences of possible capitalized expressions
                  // lowercased
                  ((int[])capExpressions.get(word.toUpperCase()))[2]++;
              }
              if (capExpressions.containsKey(word.substring(0,1)
                 .toUpperCase()+word.substring(1).toLowerCase())) {
                  // occurrences of possible capitalized expressions
                  // lowercased
                  ((int[])capExpressions
                   .get(word.substring(0,1).toUpperCase()
                        +word.substring(1).toLowerCase()))[2]++;
              }
          }

          // start the next match at the second token
          start = matcher3Tokens.start(2);
      }

      fireProgressChanged(90);
      // we do not need anymore the corpusInOneDocument document and file
      Factory.deleteResource(document);
      try {
          (new File(new URI(document.getSourceUrl().toString()))).delete();
      } catch (Exception ex) {
          ex.printStackTrace();
          return;
      }

      try {
      br =  new BufferedReader(new FileReader(
                             new File(new URI(lexiconFileURL.toString()))));

      lineContent = br.readLine();
      while (lineContent != null) {
          // read words in lexicon and store them into a hashtable
          // trim spaces and get only the first word on each line
          lexicon.put(lineContent.trim().split("\\p{Space}")[0], null);
          lineContent = br.readLine();
      }
      br.close();

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }

      iterator = abbreviations.keySet().iterator();
      while (iterator.hasNext()) {
          word = (String)iterator.next();

          // number of different words before possible abbreviation word
          ((int[])abbreviations.get(word))[3] =
              ((java.util.HashMap)previousWordsA.get(word)).size();

          // number of different words after possible abbreviation word
          ((int[])abbreviations.get(word))[4] =
              ((java.util.HashMap)nextWordsA.get(word)).size();

          // word belongs to lexicon
          array = word.split("[&_|~#^=+*@{}[-]]");
          // case of a compound word, we test all words
          if (array.length > 1) {
              for (int i = 0; i < array.length; i++) {
                  if (((String)array[i]).length() > 2
                      && (lexicon.containsKey(((String)array[i]))
                       || lexicon.containsKey(((String)array[i])
                                              .toLowerCase()))) {
                      ((int[])abbreviations.get(word))[5] = 1;
                      break;
                  }
              }
          } else {
              if (lexicon.containsKey(word)
               || lexicon.containsKey(word.toLowerCase())) {
                  ((int[])abbreviations.get(word))[5] = 1;
              }
          }
      }

      iterator = capExpressions.keySet().iterator();
      while (iterator.hasNext()) {
          word = (String)iterator.next();

          // number of different words before possible capitalized word
          ((int[])capExpressions.get(word))[3] =
             ((java.util.HashMap)previousWordsCE.get(word)).size();

          // number of different words after possible capitalized word
          ((int[])capExpressions.get(word))[4] =
              ((java.util.HashMap)nextWordsCE.get(word)).size();

          // word belongs to lexicon
          array = word.split("[&_|~#^=+*@{}[-]]");
          // case of a compound word, we test all words
          if (array.length > 1) {
              for (int i = 0; i < array.length; i++) {
                  if (((String)array[i]).length() > 2
                      && lexicon.containsKey(((String)array[i])
                                             .toLowerCase())) {
                      ((int[])capExpressions.get(word))[6] = 1;
                      break;
                  }
              }
          } else {
              if (lexicon.containsKey(word.toLowerCase())) {
                  ((int[])capExpressions.get(word))[6] = 1;
              }
          }
      }

      try {
      path = new URL(transducerDirectoryURL+"/abbreviations_stats.csv");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                      new File(new URI(path.toString())))));

      out.println("word;dot in unambiguous position;dot in ambiguous "+
                  "position;words without dot;different words before;"+
                  "different words after;belongs to lexicon");
      iterator = abbreviations.keySet().iterator();
      while (iterator.hasNext()) {
          word = (String)iterator.next();
          // write statistics about abbreviations
          out.println(word+";"+
                      ((int[])abbreviations.get(word))[0]+";"+
                      ((int[])abbreviations.get(word))[1]+";"+
                      ((int[])abbreviations.get(word))[2]+";"+
                      ((int[])abbreviations.get(word))[3]+";"+
                      ((int[])abbreviations.get(word))[4]+";"+
                      ((int[])abbreviations.get(word))[5]);
      }
      out.close();

      path = new URL(transducerDirectoryURL+"/capitalized_words_stats.csv");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                      new File(new URI(path.toString())))));

      out.println("word;capitalized expressions in unambiguous position;"+
                  "capitalized expressions in ambiguous position;"+
                  "capitalized expressions lowercased;different words "+
                  "before;different words after;capitalized expressions "+
                  "containing this word;belongs to lexicon");
      iterator = capExpressions.keySet().iterator();
      while (iterator.hasNext()) {
          word = (String)iterator.next();

          // write statistics about capitalized expressions
          out.println(word+";"+
                      ((int[])capExpressions.get(word))[0]+";"+
                      ((int[])capExpressions.get(word))[1]+";"+
                      ((int[])capExpressions.get(word))[2]+";"+
                      ((int[])capExpressions.get(word))[3]+";"+
                      ((int[])capExpressions.get(word))[4]+";"+
                      ((int[])capExpressions.get(word))[5]+";"+
                      ((int[])capExpressions.get(word))[6]);
      }
      out.close();
      fireProgressChanged(100);
      fireProcessFinished();

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 02.1.
   *
   * Apply ANNIE and custom gazetteers to the document.
   */
  private void step02_1() {

      FeatureMap params;
      FeatureMap features;

      // initialize the gazetteer
      fireStatusChanged("Initialization of Gazetteer");
      params = Factory.newFeatureMap();
      if (gazetteerFileURL != null) {
          params.put(DefaultGazetteer.DEF_GAZ_LISTS_URL_PARAMETER_NAME,
                     gazetteerFileURL);
      }
      params.put(DefaultGazetteer.DEF_GAZ_ENCODING_PARAMETER_NAME,
                 encoding);
      features = Factory.newFeatureMap();
      Gate.setHiddenAttribute(features, true);

      try {
      gazetteer = (DefaultGazetteer)
          Factory.createResource("gate.creole.gazetteer.DefaultGazetteer",
                                 params, features);
      gazetteer.setName("Gazetteer " + System.currentTimeMillis());

      // finish initialization of the gazetteer
      params = Factory.newFeatureMap();
      params.put(DefaultGazetteer.DEF_GAZ_DOCUMENT_PARAMETER_NAME,document);
      params.put(DefaultGazetteer.DEF_GAZ_ANNOT_SET_PARAMETER_NAME,
                 inputASName);
      gazetteer.setParameterValues(params);

      // run ANNIE
      gate.corpora.CorpusImpl corpus_temp = new gate.corpora.CorpusImpl();
      corpus_temp.add(document);
      annieController.setCorpus(corpus_temp);

      pListener = new IntervalProgressListener(0, 100);
      annieController.addProgressListener(pListener);
      annieController.addStatusListener(sListener);
      annieController.execute();
      annieController.removeProgressListener(pListener);
      annieController.removeStatusListener(sListener);

      // run custom gazetteer
      fireStatusChanged("Run the Gazetteer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException(
                    "The execution of the \"" + getName() +
                    "\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      gazetteer.addProgressListener(pListener);
      gazetteer.addStatusListener(sListener);
      gazetteer.execute();
      gazetteer.removeProgressListener(pListener);
      gazetteer.removeStatusListener(sListener);

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 02.2.
   *
   * Tag the compound abbreviations and uppercased letters. Tag the known
   * abbreviations thanks to the previous gazetteer as certain
   * abbreviations, tag words followed by a dot then a space as possible
   * abbreviations.
   *
   */
  private void step02_2() {

      String fileName;
      String grammar;

      // needed in order to skip lines tagged with lineAnnotationsToSkip
      // and to write back the initial annotations at the end
      document.getAnnotations()
          .addAll(document.getAnnotations("Original markups"));

      try {
      if (transducer02_2 == null) {
      // create transducer
      fileName = "segmentation02_2.jape";
      grammar =
"Phase: Step_2_2\n"+
"Input: "+lineAnnotationsToSkip+" Token SpaceToken Lookup\n"+
"Options: control = appelt\n"+
"\n"+
"Rule: Step_2_2_0\n"+
"\n"+
"{SpaceToken}({"+lineAnnotationsToSkipRule+"}){SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{}\n"+
"\n"+
"Rule: Step_2_2_00\n"+
"\n"+
"({Lookup.majorType == splitter_anti-abbreviation}\n"+
"{Token.string == \".\"}):word{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"anti-abbreviation lexicon (2.2.00)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrImpossible\", features);\n"+
"}\n"+
"\n"+
"Rule: Step_2_2_7\n"+
"\n"+
"({Lookup.majorType == splitter_abbreviation}\n"+
"{Token.string == \".\"}):word{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"abbreviation lexicon rule (2.2.7)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
"}\n"+
"\n"+
"Rule: Step_2_2_1\n"+
"\n"+
"(({Token.kind == word}{Token.string == \".\"})?\n"+
" ({Token.kind == word}{Token.string == \".\"}"+
"   ({Token.string == \"-\"})?)?\n"+
" {Token.kind == word}{Token.string == \".\"}({Token.string == \"-\"})?\n"+
" {Token.kind == word}\n"+
"{Token.string == \".\"}):word{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"compound abbreviation rule (2.2.1)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
"}\n"+
"\n"+
"Rule: Step_2_2_2\n"+
"\n"+
"(({Token.orth == allCaps})?\n"+
"  {Token.orth == allCaps}\n"+
"  {Token.kind == word}\n"+
"{Token.string == \".\"}):word{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"allcaps + word rule (2.2.2)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrPossible\", features);\n"+
"}\n"+
"\n"+
"Rule: Step_2_2_3\n"+
"\n"+
"({Token.orth == upperInitial, Token.length == \"1\"}\n"+
"{Token.string == \".\"}):word1({SpaceToken.kind == space})+\n"+
"({Token.orth == upperInitial, Token.length == \"1\"}\n"+
"{Token.string == \".\"}):word2{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word1\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"multi-initials abbreviation rule (2.2.3)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
"AS = (gate.AnnotationSet)bindings.get(\"word2\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
"}\n"+
"\n"+
// "Rule: Step_2_2_4\n"+
// "\n"+
// "{Token.orth == upperInitial}({Token.string == \".\"})?\n"+
// "({SpaceToken.kind == space})+\n"+
// "({Token.orth == upperInitial, Token.length == \"1\"}\n"+
// "{Token.string == \".\"}):word1({SpaceToken.kind == space})+\n"+
// "(({Token.orth == upperInitial, Token.length == \"1\"}\n"+
// "{Token.string == \".\"}):word2({SpaceToken.kind == space})+)?\n"+
// "{Token.orth == upperInitial}\n"+
// "\n"+
// "-->\n"+
// "\n"+
// "{\n"+
// "gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word1\");\n"+
// "gate.FeatureMap features = Factory.newFeatureMap();\n"+
// "features.put(\"rule\", \"person initial abbreviation rule (2.2.4)\");\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
// "AS = (gate.AnnotationSet)bindings.get(\"word2\");\n"+
// "if (AS != null) {\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
// "}\n"+
// "}\n"+
// "\n"+
// "Rule: Step_2_2_5\n"+
// "\n"+
// "({Token.string == \",\"}|{Token.string == \";\"}|\n"+
// "{Token.string == \":\"}|{Token.string == \"and\"}|\n"+
// "{Token.string == \"from\"}|{Token.string == \"of\"}|\n"+
// "{Token.string == \"by\"})({SpaceToken.kind == space})+\n"+
// "({Token.orth == upperInitial, Token.length == \"1\"}\n"+
// "{Token.string == \".\"}):word({SpaceToken.kind == space})+\n"+
// "({Token.orth == upperInitial}|{Token.orth == lowercase})\n"+
// "\n"+
// "-->\n"+
// "\n"+
// "{\n"+
// "gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
// "gate.FeatureMap features = Factory.newFeatureMap();\n"+
// "features.put(\"rule\", \"person abbreviation rule (2.2.5)\");\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrCertain\", features);\n"+
// "}\n"+
// "\n"+
"Rule: Step_2_2_6\n"+
"\n"+
"(({Token.kind == number})*({Token.string == \".\"})?"+
"({Token.kind == number})+({Token.string == \"%\"}"+
"|{Token.string == \"s\"})?\n"+
"{Token.string == \".\"}):word{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"number rule (2.2.6)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrImpossible\", features);\n"+
"}\n"+
"\n"+
"Rule: Step_2_2_8\n"+
"\n"+
"(({Token.kind == word}|{Token.kind == number})+\n"+
"{Token.string == \".\"}):word{SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"default abbreviation possible rule (2.2.8)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"AbbrPossible\",features);\n"+
"}\n";
      transducer02_2 = createTransducer(grammar, document, inputASName,
                                        inputASName, fileName, encoding);
      } else {
          transducer02_2.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer02_2.addProgressListener(pListener);
      transducer02_2.addStatusListener(sListener);
      transducer02_2.execute();
      transducer02_2.removeProgressListener(pListener);
      transducer02_2.removeStatusListener(sListener);

      // user define transducer 1
      fileName = "user_abbr_1.jape";
      if ((new File(new URI((new URL(transducerDirectoryURL+"/"+fileName))
                            .toString()))).exists()) {
      if (transducer02_2_1 == null) {
      // create transducer
      transducer02_2_1 = createTransducer(null, document, inputASName,
                                          inputASName, fileName, encoding);
      } else {
          transducer02_2_1.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer02_2_1.addProgressListener(pListener);
      transducer02_2_1.addStatusListener(sListener);
      transducer02_2_1.execute();
      transducer02_2_1.removeProgressListener(pListener);
      transducer02_2_1.removeStatusListener(sListener);
      }

      // user define transducer 2
      fileName = "user_abbr_2.jape";
      if ((new File(new URI((new URL(transducerDirectoryURL+"/"+fileName))
                            .toString()))).exists()) {
      if (transducer02_2_2 == null) {
      // create transducer
      transducer02_2_2 = createTransducer(null, document, inputASName,
                                          inputASName, fileName, encoding);
      } else {
          transducer02_2_2.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer02_2_2.addProgressListener(pListener);
      transducer02_2_2.addStatusListener(sListener);
      transducer02_2_2.execute();
      transducer02_2_2.removeProgressListener(pListener);
      transducer02_2_2.removeStatusListener(sListener);
      }

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 2.4.
   *
   * Use previous statistics to tag possible abbreviations as certain or
   * impossible abbreviations.
   *
   * @param min minimum number of occurrences in statistics needed to use
   * them
   * @param ratio ratio between occurrences with dot and without dot to
   * class abbreviations in certain or impossible classes
   * @param range interval of the window of possible abbreviation knowing
   * that below it is certain and up it is impossible class
   */
  private void step02_4(double min, double ratio, double range) {

      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.Annotation A;
      gate.FeatureMap features;
      int start, end;
      double dot_u, dot_a, no_dot;
      String s, num;
      boolean lexicon;
      Object[] array;

      // iterate on all abbreviations
      HashSet AbbrAS = new HashSet();
      AbbrAS.add("AbbrPossible");
      if (!automatic) {
          AbbrAS.add("AbbrCertain");
          AbbrAS.add("AbbrImpossible");
      }
      aAS = AS.get(AbbrAS);
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end-1);
          // delete characters after a '
          s = s.replaceAll("\'.*$", "");

          if (!abbreviations.containsKey(s)) { continue; }

          // add statistics to tag
          features = A.getFeatures();
          features.put("ending_with_dot_unambiguous",
                       String.valueOf(((int[])abbreviations.get(s))[0]));
          features.put("ending_with_dot_ambiguous",
                       String.valueOf(((int[])abbreviations.get(s))[1]));
          features.put("not_ending_with_dot",
                       String.valueOf(((int[])abbreviations.get(s))[2]));
          features.put("different_words_before",
                       String.valueOf(((int[])abbreviations.get(s))[3]));
          features.put("different_words_after",
                       String.valueOf(((int[])abbreviations.get(s))[4]));
          features.put("belongs_to_lexicon",
                      (((int[])abbreviations.get(s))[5]==1)?"true":"false");
      }
      }

      // iterate on all AbbrPossible
      aAS = AS.get("AbbrPossible");
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end-1);
          s = s.replaceAll("\'.*$", "");

          if (!abbreviations.containsKey(s)) {

              // impossible abbreviation
              if (s.length() >= 10
              || (s.toUpperCase().equals(s) && s.length() >= 3)) {

                  features = A.getFeatures();
                  AS.remove(A);
                  features.put("rule", "no statistics but long word or "+
                               "upcase rule (2.4.1)");
                  AS.add(A.getStartNode(), A.getEndNode(),
                         "AbbrImpossible", features);
              }
              continue;
          }

          features = A.getFeatures();

          // get statistics for this word
          dot_u = ((int[])abbreviations.get(s))[0];
          dot_a = ((int[])abbreviations.get(s))[1];
          no_dot = ((int[])abbreviations.get(s))[2];
          lexicon = (((int[])abbreviations.get(s))[5]==1) ? true : false;

          num = " x "+new java.text.DecimalFormat("#0.0")
              .format(ratio-(ratio*(range/100)))+" ";

          // impossible abbreviation
          if (s.length() >= 10
          || (s.length() >= 5 && lexicon)
          || (s.toUpperCase().equals(s) && s.length() >= 3)) {

              AS.remove(A);
              features.put("rule",
                           "long word, upcase and lexicon rule (2.4.2)");
              AS.add(A.getStartNode(), A.getEndNode(),
                     "AbbrImpossible", features);

          // possible or impossible abbreviation
          } else if ((dot_u + dot_a + no_dot) < min) {

              if (lexicon) {

                  // the word is common word
                  AS.remove(A);
                  features.put("rule", "common words lexicon (2.4.3)");
                  AS.add(A.getStartNode(), A.getEndNode(),
                         "AbbrImpossible", features);
              }
              // else let it tagged possible abbreviation

          // impossible abbreviation
          } else if ((dot_u + dot_a) < ( (ratio-(ratio*(range/100)))
                                 * ((no_dot==0)?1:no_dot) )) {

              AS.remove(A);
              features.put("rule",
                     "more statistics without dot"+num+"than with (2.4.4)");
              AS.add(A.getStartNode(), A.getEndNode(),
                     "AbbrImpossible", features);

          // certain abbreviation
          } else if ((dot_u + dot_a) > ( (ratio-(ratio*(range/100)))
                                 * ((no_dot==0)?1:no_dot) )) {

              AS.remove(A);
              features.put("rule",
                     "more statistics with dot than without"+num+"(2.4.5)");
              AS.add(A.getStartNode(), A.getEndNode(),
                     "AbbrCertain", features);

          // possible or impossible abbreviation
          } else if (lexicon) {

                  // the word is common word
                  AS.remove(A);
                  features.put("rule", "common words lexicon (2.4.6)");
                  AS.add(A.getStartNode(), A.getEndNode(),
                         "AbbrImpossible", features);
          }
      }
      }

      if (docEditor != null) {
          docEditor.setTypeSelected(null, "AbbrCertain", true);
          docEditor.setTypeSelected(null, "AbbrPossible", true);
          docEditor.setTypeSelected(null, "AbbrImpossible", true);
      }
  }

  /**
   * Step 03.1.
   *
   * Add features to certain/possible abbreviations as a new tag :
   * Abbreviation. String prefixes, length of word, case, previous and next
   * word Token.category and Token.string full or prefixes, confidence of
   * beeing abbreviation, etc.
   */
  private void step03_1() {

      String fileName;
      String grammar;
      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.FeatureMap features;
      gate.Annotation A;
      int start, end;
      String s;
      Object[] array;

      // iterate on all abbreviations
      HashSet AbbrAS = new HashSet();
      AbbrAS.add("AbbrPossible");
      if (!automatic) {
          AbbrAS.add("AbbrCertain");
          AbbrAS.add("AbbrImpossible");
      }
      aAS = AS.get(AbbrAS);
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end-1);
          s = s.replaceAll("\'.*$", "");
          features = A.getFeatures();
          if (A.getType().equals("AbbrPossible")) {
              features.put("previous_class", "possible");
              features.put("class", "possible");
          } else if (A.getType().equals("AbbrCertain")) {
              features.put("previous_class", "certain");
              features.put("class", "certain");
          } else {
              features.put("previous_class", "impossible");
              features.put("class", "impossible");
          }
          features.put("length", new Integer(s.length()).toString());
          if (s.toUpperCase().equals(s)) {
              features.put("case", "up");
          } else if (s.toLowerCase().equals(s)) {
              features.put("case", "low");
          } else if (s.substring(0,1).toUpperCase().equals(
                       s.substring(0,1))
                  && s.substring(1).toLowerCase().equals(
                       s.substring(1))) {
              features.put("case", "cap");
          } else {
              features.put("case", "mix");
          }
          if (s.matches(".*[0-9].*")) {
              features.put("contains_digit", "true");
          } else {
              features.put("contains_digit", "false");
          }
          AS.add(A.getStartNode(), A.getEndNode(), "Abbreviation",
                 features);
      }
      }

      // add previous and next word Token.category and token.string as
      // feature to Abbreviation tagged words

      try {
      if (transducer03_1 == null) {
      // create transducer
      fileName = "segmentation03_1.jape";
      grammar =
"Phase: Step_3_1\n"+
"Input: Token SpaceToken Abbreviation\n"+
"Options: control = appelt\n"+
"\n"+
"Rule: Step_3_1_1\n"+
"\n"+
"(({Token.kind == number}\n"+
"|({Token.kind == word}({Token.string == \".\"})?)):previous_word\n"+
"({SpaceToken}|{Token.string == \"_\"}))?\n"+
"({Abbreviation}):word\n"+
"({SpaceToken}\n"+
"({Token.kind == number}\n"+
"|({Token.kind == word}({Token.string == \".\"})?)):next_word)?\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.AnnotationSet pAS = (gate.AnnotationSet)bindings.get(\"previous_word\");\n"+
"gate.AnnotationSet nAS = (gate.AnnotationSet)bindings.get(\"next_word\");\n"+
"gate.Annotation A = (gate.Annotation)AS.iterator().next();\n"+
"gate.FeatureMap features = A.getFeatures();\n"+
"String s;\n"+
"if (pAS != null) {\n"+
"gate.Annotation pA = (gate.Annotation)pAS.iterator().next();\n"+
"s = (String)pA.getFeatures().get(\"string\");\n"+
"features.put(\"previous_string\", s);\n";
if (POSTagging) {
grammar +=
"features.put(\"previous_category\", pA.getFeatures().get(\"category\"));\n";
}
grammar +=
"if (s.endsWith(\".\")) {\n"+
"features.put(\"previous_ends_with_dot\", \"true\");\n"+
"} else {\n"+
"features.put(\"previous_ends_with_dot\", \"false\");\n"+
"}\n"+
"if (s.toUpperCase().equals(s)) {\n"+
"    features.put(\"previous_case\", \"up\");\n"+
"} else if (s.toLowerCase().equals(s)) {\n"+
"    features.put(\"previous_case\", \"low\");\n"+
"} else if (s.substring(0,1).toUpperCase().equals(\n"+
"             s.substring(0,1))\n"+
"        && s.substring(1).toLowerCase().equals(\n"+
"             s.substring(1))) {\n"+
"    features.put(\"previous_case\", \"cap\");\n"+
"} else {\n"+
"    features.put(\"previous_case\", \"mix\");\n"+
"}\n"+
"}\n"+
"if (nAS != null) {\n"+
"gate.Annotation nA = (gate.Annotation)nAS.iterator().next();\n"+
"s = (String)nA.getFeatures().get(\"string\");\n"+
"features.put(\"next_string\", s);\n";
if (POSTagging) {
grammar +=
"features.put(\"next_category\", nA.getFeatures().get(\"category\"));\n";
}
grammar +=
"if (s.endsWith(\".\")) {\n"+
"features.put(\"next_ends_with_dot\", \"true\");\n"+
"} else {\n"+
"features.put(\"next_ends_with_dot\", \"false\");\n"+
"}\n"+
"if (s.toUpperCase().equals(s)) {\n"+
"    features.put(\"next_case\", \"up\");\n"+
"} else if (s.toLowerCase().equals(s)) {\n"+
"    features.put(\"next_case\", \"low\");\n"+
"} else if (s.substring(0,1).toUpperCase().equals(\n"+
"             s.substring(0,1))\n"+
"        && s.substring(1).toLowerCase().equals(\n"+
"             s.substring(1))) {\n"+
"    features.put(\"next_case\", \"cap\");\n"+
"} else {\n"+
"    features.put(\"next_case\", \"mix\");\n"+
"}\n"+
"}\n"+
"}\n";
      transducer03_1 = createTransducer(grammar, document, inputASName,
                                        inputASName, fileName, encoding);
      } else {
          transducer03_1.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer03_1.addProgressListener(pListener);
      transducer03_1.addStatusListener(sListener);
      transducer03_1.execute();
      transducer03_1.removeProgressListener(pListener);
      transducer03_1.removeStatusListener(sListener);

      } catch (ExecutionException e) {
          e.printStackTrace();
          return;
      }

      // iterate on all Abbreviation
      aAS = AS.get("Abbreviation");

      // add some features
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          features = A.getFeatures();
          s = (String)features.get("previous_string");
          if (s != null) {
          if (s.length() >= 6 && prefixes.containsKey(s.substring(0,6))) {
              features.put("previous_prefix", s.substring(0,6));
          } else if (s.length() >= 5
                     && prefixes.containsKey(s.substring(0,5))) {
              features.put("previous_prefix", s.substring(0,5));
          } else if (s.length() >= 4
                     && prefixes.containsKey(s.substring(0,4))) {
              features.put("previous_prefix", s.substring(0,4));
          } else if (s.length() >= 3
                     && prefixes.containsKey(s.substring(0,3))) {
              features.put("previous_prefix", s.substring(0,3));
          } else if (s.length() >= 2
                     && prefixes.containsKey(s.substring(0,2))) {
              features.put("previous_prefix", s.substring(0,2));
          } else if (s.length() >= 1
                     && prefixes.containsKey(s.substring(0,1))) {
              features.put("previous_prefix", s.substring(0,1));
          }
          if (s.length() >= 6
              && suffixes.containsKey(s.substring(s.length()-6))) {
              features.put("previous_suffix", s.substring(s.length()-6));
          } else if (s.length() >= 5
                     && suffixes.containsKey(s.substring(s.length()-5))) {
              features.put("previous_suffix", s.substring(s.length()-5));
          } else if (s.length() >= 4
                     && suffixes.containsKey(s.substring(s.length()-4))) {
              features.put("previous_suffix", s.substring(s.length()-4));
          } else if (s.length() >= 3
                     && suffixes.containsKey(s.substring(s.length()-3))) {
              features.put("previous_suffix", s.substring(s.length()-3));
          } else if (s.length() >= 2
                     && suffixes.containsKey(s.substring(s.length()-2))) {
              features.put("previous_suffix", s.substring(s.length()-2));
          } else if (s.length() >= 1
                     && suffixes.containsKey(s.substring(s.length()-1))) {
              features.put("previous_suffix", s.substring(s.length()-1));
          }
          }
          s = (String)features.get("next_string");
          if (s != null) {
          if (s.length() >= 6 && prefixes.containsKey(s.substring(0,6))) {
              features.put("next_prefix", s.substring(0,6));
          } else if (s.length() >= 5
                     && prefixes.containsKey(s.substring(0,5))) {
              features.put("next_prefix", s.substring(0,5));
          } else if (s.length() >= 4
                     && prefixes.containsKey(s.substring(0,4))) {
              features.put("next_prefix", s.substring(0,4));
          } else if (s.length() >= 3
                     && prefixes.containsKey(s.substring(0,3))) {
              features.put("next_prefix", s.substring(0,3));
          } else if (s.length() >= 2
                     && prefixes.containsKey(s.substring(0,2))) {
              features.put("next_prefix", s.substring(0,2));
          } else if (s.length() >= 1
                     && prefixes.containsKey(s.substring(0,1))) {
              features.put("next_prefix", s.substring(0,1));
          }
          if (s.length() >= 6
              && suffixes.containsKey(s.substring(s.length()-6))) {
              features.put("next_suffix", s.substring(s.length()-6));
          } else if (s.length() >= 5
                     && suffixes.containsKey(s.substring(s.length()-5))) {
              features.put("next_suffix", s.substring(s.length()-5));
          } else if (s.length() >= 4
                     && suffixes.containsKey(s.substring(s.length()-4))) {
              features.put("next_suffix", s.substring(s.length()-4));
          } else if (s.length() >= 3
                     && suffixes.containsKey(s.substring(s.length()-3))) {
              features.put("next_suffix", s.substring(s.length()-3));
          } else if (s.length() >= 2
                     && suffixes.containsKey(s.substring(s.length()-2))) {
              features.put("next_suffix", s.substring(s.length()-2));
          } else if (s.length() >= 1
                     && suffixes.containsKey(s.substring(s.length()-1))) {
              features.put("next_suffix", s.substring(s.length()-1));
          }
          }
      }
      }

      if (docEditor != null) {
          docEditor.setTypeSelected(null, "AbbrCertain", false);
          docEditor.setTypeSelected(null, "AbbrPossible", false);
          docEditor.setTypeSelected(null, "AbbrImpossible", false);
          docEditor.setTypeSelected(null, "Abbreviation", true);
      }
  }

  /**
   * Step 03.2.
   *
   * Learn a model on these Abbreviation tagged words.
   *
   * Train a weka tree machine learning on the features of abbreviations to
   * learn the feature class abbreviation with values certain, impossible
   * and possible.
   *
   * @parameter features array that define if a feature is enable, value 1,
   * or disable, value 0
   * @returns a vector of vector of String or String that represent a table
   * of results, rows first
   */
  private Vector step03_2(int[] features) {

      String fileName;
      PrintWriter out;
      URL newConfigFileURL;
      String config;
      Iterator iterator;
      String prefixes_values, suffixes_values, tags_values;
      MLEngine engine;
      Vector message, line;

      prefixes_values = "";
      iterator = prefixes.keySet().iterator();
      while (iterator.hasNext()) {
          prefixes_values += "<VALUE>"+(String)iterator.next()+"</VALUE>\n";
      } 

      suffixes_values = "";
      iterator = suffixes.keySet().iterator();
      while (iterator.hasNext()) {
          suffixes_values += "<VALUE>"+(String)iterator.next()+"</VALUE>\n";
      } 

      tags_values = "";
      iterator = tags.keySet().iterator();
      while (iterator.hasNext()) {
          tags_values += "<VALUE>"+(String)iterator.next()+"</VALUE>\n";
      } 

      // create ML configuration
      fileName = "ml_3_2.xml";
      config =
"<?xml version=\"1.0\" encoding=\""+encoding+"\"?>\n"+
"<ML-CONFIG>\n"+
"  <DATASET>\n"+
"  <INSTANCE-TYPE>Abbreviation</INSTANCE-TYPE>\n"+
"\n";
if (features[0] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.ending_with_dot_unambiguous(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>ending_with_dot_unambiguous</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[1] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.ending_with_dot_ambiguous(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>ending_with_dot_ambiguous</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[2] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.not_ending_with_dot(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>not_ending_with_dot</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[3] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.different_words_before(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>different_words_before</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[4] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.different_words_after(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>different_words_after</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[5] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.belongs_to_lexicon(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>belongs_to_lexicon</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[6] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.length(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>length</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[7] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.case(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>case</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>low</VALUE>\n"+
"      <VALUE>up</VALUE>\n"+
"      <VALUE>cap</VALUE>\n"+
"      <VALUE>mix</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[8] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.contains_digit(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>contains_digit</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[9] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.previous_category(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>previous_category</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
tags_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[10] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.previous_ends_with_dot(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>previous_ends_with_dot</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[11] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.previous_case(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>previous_case</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>low</VALUE>\n"+
"      <VALUE>up</VALUE>\n"+
"      <VALUE>cap</VALUE>\n"+
"      <VALUE>mix</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[12] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.previous_prefix(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>previous_prefix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
prefixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[13] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.previous_suffix(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>previous_suffix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
suffixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[14] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.next_category(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>next_category</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
tags_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[15] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.next_ends_with_dot(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>next_ends_with_dot</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[16] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.next_case(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>next_case</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>low</VALUE>\n"+
"      <VALUE>up</VALUE>\n"+
"      <VALUE>cap</VALUE>\n"+
"      <VALUE>mix</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[17] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.next_prefix(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>next_prefix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
prefixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[18] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.next_suffix(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>next_suffix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
suffixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[19] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.user_abbr_1(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>user_abbr_1</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[20] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.user_abbr_2(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>user_abbr_2</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
config +=
"  <ATTRIBUTE>\n"+
"    <NAME>Abbreviation.class(0)</NAME>\n"+
"    <TYPE>Abbreviation</TYPE>\n"+
"    <FEATURE>class</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>certain</VALUE>\n"+
"      <VALUE>impossible</VALUE>\n"+
"      <VALUE>possible</VALUE>\n"+ 
"    </VALUES>\n"+
"    <CLASS/>\n"+
"  </ATTRIBUTE>\n"+
"\n"+
"  </DATASET>\n"+
"\n"+
"  <ENGINE>\n"+
"    <WRAPPER>gate.creole.ml.weka.Wrapper</WRAPPER>\n"+
"    <OPTIONS>\n"+
"        <CLASSIFIER OPTIONS=\"-M 4\">weka.classifiers.trees.J48</CLASSIFIER>\n"+
"    </OPTIONS>\n"+
"  </ENGINE>\n"+
"</ML-CONFIG>";
      try {
      newConfigFileURL = new URL(transducerDirectoryURL+"/"+fileName);
      out = new PrintWriter(new BufferedWriter(
           new FileWriter(new File(new URI(newConfigFileURL.toString())))));
      out.println(config);
      out.close();

      // create ML PR
      abbml = new MachineLearningPR();
      abbml.setConfigFileURL(newConfigFileURL);
      abbml.setDocument(document);
      abbml.init();

      // run ML PR in training mode
      abbml.setTraining(new Boolean(true));
      abbml.execute();

      // save data
      engine = abbml.getEngine();
      FileWriter fw = new FileWriter(
                 (new URL(transducerDirectoryURL+"/abbml.arff")).getPath());
      ((gate.creole.ml.weka.Wrapper)engine).saveDatasetAsARFF(fw);
      fw.close();

      // first apply a tree classifier
      message = new Vector();
      line = new Vector();
      Classifier c = new weka.classifiers.trees.J48();
      FileReader fr = new FileReader(
                 (new URL(transducerDirectoryURL+"/abbml.arff")).getPath());
      weka.core.Instances instances = new weka.core.Instances(fr);
      instances.setClassIndex(instances.numAttributes() - 1);
      // train classifier on all instances
      ((weka.classifiers.trees.J48)c).setSaveInstanceData(true);
      c.buildClassifier(instances);
      // test classifier on all instances
      EvaluationUtils ev = new EvaluationUtils();
      weka.core.FastVector fv = ev.getTestPredictions(c, instances);

      java.util.Enumeration predictions = fv.elements();
      weka.classifiers.evaluation.Prediction prediction;
      weka.core.Instances instances_unpredicted =
          new weka.core.Instances(instances, 0);
      message.add("C4.5 Unpredicted Instances");
      message.add("");
      Object[] annotations =
          document.getAnnotations().get("Abbreviation").toArray();
      java.util.Comparator comparator = new java.util.Comparator() {
              public int compare(Object o1, Object o2) {
                  return
                      (((SimpleAnnotation)o1).getStartNode().getOffset()
            .compareTo(((SimpleAnnotation)o2).getStartNode().getOffset()));
              }
          };
      // sort annotations the same way as predictions and instances
      java.util.Arrays.sort(annotations, comparator);
      int start, end;
      String rule;
      String word, context;
      String[] elements;
      gate.Annotation ann;
      Vector instances_unpredicted_context = new Vector();
      Vector instances_unpredicted_word = new Vector();
      for (int i = 0; predictions.hasMoreElements(); i++) {
          prediction = (weka.classifiers.evaluation.Prediction)
              predictions.nextElement();

          // if prediction failed for this instance
          if (prediction.actual() != prediction.predicted()) {

              // get text of this instance and context
              line = new Vector();
              ann = (Annotation)annotations[i];
              start = ann.getStartNode().getOffset().intValue();
              end = ann.getEndNode().getOffset().intValue();
              word = document.getContent().toString().substring(start, end);
              start -= 20; end += 20;
              if (start < 0) { start = 0; }
              if (end >= document.getContent().toString().length()) {
                  end = document.getContent().toString().length()-1; }
              context = document.getContent().toString()
                  .substring(start, end).replaceAll("\\p{Space}+", " ");
              line.add(word);
              instances_unpredicted_word.add(word);
              line.add("[...] "+context+" [...]");
              instances_unpredicted_context.add("[...] "+context+" [...]");
              // get features of this instance
              elements = instances.instance(i).toString().trim().split(",");
              for (int j = 0; j < elements.length; j++) {
                  line.add(elements[j]);
              }
              switch (new Double(prediction.predicted()).intValue()) {
              case 0 : line.add("certain"); break;
              case 1 : line.add("impossible"); break;
              case 2 : line.add("possible"); break;}
              rule = (ann.getFeatures().get("rule") != null)?
                  (String)ann.getFeatures().get("rule"):"";
              if (!ann.getFeatures().get("previous_class")
                  .equals(ann.getFeatures().get("class"))) {
                  line.add("user");
              } else if (prediction.actual() == 2) {
                  line.add("no process");
              } else {
                  line.add(rule);
              }
              message.add(line);
              instances_unpredicted.add(instances.instance(i));
          }
      }
      message.add("");
      message.add(instances_unpredicted.numInstances()+
                  " unpredicted on "+instances.numInstances()+
                  " ("+new java.text.DecimalFormat("#0.00")
                  .format(100.00*instances_unpredicted.numInstances()
                          /instances.numInstances())+"%)");

      // second apply a rule classifier on previous unpredicted instances
      message.add(""); message.add("");
      message.add("JRip Unpredicted Instances");
      message.add("");
      Classifier c2 = new weka.classifiers.rules.JRip();
      try {
          fv = ev.getTrainTestPredictions(c2, instances_unpredicted,
                                          instances_unpredicted);
      } catch (Exception e) {
          // not enough data to build the classifier
          message.add(e.getMessage());
          return message;
      }

      predictions = fv.elements();
      int num = 0;
      for (int i = 0; predictions.hasMoreElements(); i++) {
          prediction = (weka.classifiers.evaluation.Prediction)
              predictions.nextElement();

          // if prediction failed for this instance
          if (prediction.actual() != prediction.predicted()) {
              line = new Vector();
              line.add(instances_unpredicted_word.get(i));
              line.add(instances_unpredicted_context.get(i));
              // get features of this instance
              elements = instances_unpredicted.instance(i)
                  .toString().trim().split(",");
              for (int j = 0; j < elements.length; j++) {
                  line.add(elements[j]);
              }
              switch (new Double(prediction.predicted()).intValue()) {
              case 0 : line.add("certain"); break;
              case 1 : line.add("impossible"); break;
              case 2 : line.add("possible"); break;}
              message.add(line);
              num++;
          }
      }
      message.add("");
      message.add(num+" unpredicted on "
                  +instances_unpredicted.numInstances());

      // other solutions :
      //
      // JR48.toSource() and put each instance through the if-then program
      // and get the name of the last if-then method
      //
      // JR48.graph() and get the place where each instance is located

      // display tree classifier
      final javax.swing.JFrame jf = 
          new javax.swing.JFrame("Weka Classifier Tree Visualizer: J48");
      jf.setSize(500, 400);
      jf.getContentPane().setLayout(new java.awt.BorderLayout());

      weka.gui.treevisualizer.TreeVisualizer tv =
          new weka.gui.treevisualizer.TreeVisualizer(null,
                                    ((weka.classifiers.trees.J48)c).graph(),
                                  new weka.gui.treevisualizer.PlaceNode2());
      jf.getContentPane().add(tv, java.awt.BorderLayout.CENTER);
      jf.addWindowListener(new java.awt.event.WindowAdapter() {
              public void windowClosing(java.awt.event.WindowEvent e) {
                  jf.dispose();
              }
          });
      jf.setVisible(true);
      tv.fitToScreen();

      } catch (Exception e) {
          e.printStackTrace();
          return null;
      }

      return message;
  }

  /**
   * Step 04.1.
   *
   * Tag the known capitalized words thanks to the previous gazetteer as
   * certain capitalized words, tag words that are capitalized or upcased as
   * possible capitalized words.
   */
  private void step04_1() {

      String fileName;
      String grammar;

      try {
      if (transducer04_1 == null) {
      // create transducer
      fileName = "segmentation04_1.jape";
      grammar =
"Phase: Step_4_1\n"+
"Input: Token SpaceToken Lookup AbbrCertain "+lineAnnotationsToSkip+"\n"+
"Options: control = appelt\n"+
"\n"+
"Rule: Step_4_1_00\n"+
"\n"+
"{SpaceToken}({"+lineAnnotationsToSkipRule+"}){SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{}\n"+
"\n"+
"Rule: Rule_4_1_0\n"+
"\n"+
"{Token.string == \".\"}(({SpaceToken.kind == space})+{AbbrCertain}):word\n"+
"\n"+
"-->\n"+
"\n"+
"{"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"AS = AS.get(\"AbbrCertain\");\n"+
"gate.Annotation A = (gate.Annotation)AS.iterator().next();\n"+
"int start = A.getStartNode().getOffset().intValue();\n"+
"int end = A.getEndNode().getOffset().intValue();\n"+
"String s = doc.getContent().toString().substring(start, end).trim();\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"abbreviation that is a capitalized word too rule (4.1.0)\");\n"+
"try {\n"+
"if (s.substring(0,1).toUpperCase().equals(s.substring(0,1))) {\n"+
"outputAS.add(new Long(start-1), new Long(end-1), \"CapWordCertain\", features);\n"+
"} else {\n"+
"outputAS.add(new Long(start-1), new Long(end-1), \"CapWordImpossible\", features);\n"+
"}\n"+
"} catch (Exception e) {}\n"+
"}\n"+
"\n"+
"Rule: Step_4_1_1\n"+
"\n"+
"{Token.string == \".\"}(({SpaceToken.kind == space})+\n"+
"{Lookup.majorType == splitter_capitalized}):word\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"capitalized word lexicon (4.1.1)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"CapWordCertain\", features);\n"+
"}\n"+
"\n"+
"Rule: Step_4_1_2\n"+
"\n"+
"{Token.string == \".\"}(({SpaceToken.kind == space})+\n"+
"{Lookup.majorType == splitter_anti-capitalized}):word\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"anti-capitalized word lexicon (4.1.2)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"CapWordImpossible\", features);\n"+
"}\n"+
"\n"+
// "Rule: Step_4_1_3\n"+
// "\n"+
// "{Token.string == \".\"}(({SpaceToken.kind == space})+\n"+
// "{Token.kind == number}):word\n"+
// "\n"+
// "-->\n"+
// "\n"+
// "{\n"+
// "gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
// "gate.FeatureMap features = Factory.newFeatureMap();\n"+
// "features.put(\"rule\", \"number rule (4.1.3)\");\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"CapWordImpossible\", features);\n"+
// "}\n"+
// "\n"+
"Rule: Rule_4_1_4\n"+
"\n"+
"{Token.string == \".\"}(({SpaceToken.kind == space})+\n"+
"({Token.kind == word, Token.orth == allCaps}|\n"+
"{Token.kind == word, Token.orth == upperInitial})\n"+
"({Token.string == \"-\"}|{Token.string == \"_\"}|\n"+
"{Token.kind == number}|{Token.kind == word})*\n"+
"):word\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"default capitalized word possible rule (4.1.4)\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"CapWordPossible\", features);\n"+
"}\n"+
"\n";
      transducer04_1 = createTransducer(grammar, document, inputASName,
                                        inputASName, fileName, encoding);
      } else {
          transducer04_1.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer04_1.addProgressListener(pListener);
      transducer04_1.addStatusListener(sListener);
      transducer04_1.execute();
      transducer04_1.removeProgressListener(pListener);
      transducer04_1.removeStatusListener(sListener);

      // user define transducer 1
      fileName = "user_cap_1.jape";
      if ((new File(new URI((new URL(transducerDirectoryURL+"/"+fileName))
                            .toString()))).exists()) {
      if (transducer04_1_1 == null) {
      // create transducer
      transducer04_1_1 = createTransducer(null, document, inputASName,
                                          inputASName, fileName, encoding);
      } else {
          transducer04_1_1.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer04_1_1.addProgressListener(pListener);
      transducer04_1_1.addStatusListener(sListener);
      transducer04_1_1.execute();
      transducer04_1_1.removeProgressListener(pListener);
      transducer04_1_1.removeStatusListener(sListener);
      }

      // user define transducer 2
      fileName = "user_cap_2.jape";
      if ((new File(new URI((new URL(transducerDirectoryURL+"/"+fileName))
                            .toString()))).exists()) {
      if (transducer04_1_2 == null) {
      // create transducer
      transducer04_1_2 = createTransducer(null, document, inputASName,
                                          inputASName, fileName, encoding);
      } else {
          transducer04_1_2.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer04_1_2.addProgressListener(pListener);
      transducer04_1_2.addStatusListener(sListener);
      transducer04_1_2.execute();
      transducer04_1_2.removeProgressListener(pListener);
      transducer04_1_2.removeStatusListener(sListener);
      }

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 4.2.
   *
   * Use previous statistics to tag possible capitalized words as certain or
   * impossible capitalized words.
   *
   * @param min minimum number of occurrences in statistics needed to use
   * them
   * @param ratio ratio between occurrences with upper initial and without
   * to class capitalized words in certain or impossible classes
   * @param range interval of the window of possible capitalized words
   * knowing that below it is certain and up it is impossible class
   */
  private void step04_2(double min, double ratio, double range) {

      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.Annotation A;
      gate.FeatureMap features;
      int start, end;
      double up_u, up_a, no_up;
      String s, num;
      boolean lexicon;
      Object[] array;

      // iterate on all Capitalized words
      HashSet CapWordAS = new HashSet();
      CapWordAS.add("CapWordPossible");
      if (!automatic) {
          CapWordAS.add("CapWordCertain");
          CapWordAS.add("CapWordImpossible");
      }
      aAS = AS.get(CapWordAS);
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start+1, end);
          // delete ends of line, spaces and possessive in the string
          s = s.replaceAll("([\\r\\n ]|(\'.*$))", "");
          if (!capExpressions.containsKey(s)) { continue; }

          // add statistics to tag
          features = A.getFeatures();
          features.put("uppercased_initial_unambiguous",
                       String.valueOf(((int[])capExpressions.get(s))[0]));
          features.put("uppercased_initial_ambiguous",
                       String.valueOf(((int[])capExpressions.get(s))[1]));
          features.put("without_uppercased_initial",
                       String.valueOf(((int[])capExpressions.get(s))[2]));
          features.put("different_words_before",
                       String.valueOf(((int[])capExpressions.get(s))[3]));
          features.put("different_words_after",
                       String.valueOf(((int[])capExpressions.get(s))[4]));
          features.put("containing_expressions",
                       String.valueOf(((int[])capExpressions.get(s))[5]));
          features.put("belongs_to_lexicon",
                     (((int[])capExpressions.get(s))[6]==1)?"true":"false");
      }
      }

      // iterate on all CapWordPossible
      aAS = AS.get("CapWordPossible");
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start+1, end);
          s = s.replaceAll("([\\r\\n ]|(\'.*$))", "");

          if (!capExpressions.containsKey(s)) { continue; }

          features = A.getFeatures();

          // get statistics for this word
          up_u = ((int[])capExpressions.get(s))[0];
          up_a = ((int[])capExpressions.get(s))[1];
          no_up = ((int[])capExpressions.get(s))[2];
          lexicon = (((int[])capExpressions.get(s))[6]==1) ? true : false;

          num = " x "+new java.text.DecimalFormat("#0.0")
              .format(ratio-(ratio*(range/100)))+" ";

          // possible or impossible capitalized word
          if ((up_u + up_a + no_up) < min) {

              if (lexicon) {

                  // the word is common word
                  AS.remove(A);
                  features.put("rule", "common words lexicon (4.2.1)");
                  AS.add(A.getStartNode(), A.getEndNode(),
                         "CapWordImpossible", features);
              }
              // else let it tagged possible capitalized word

          // impossible capitalized word
          } else if ((up_u + up_a) < ( (ratio-(ratio*(range/100)))
                                 * ((no_up==0)?1:no_up) )) {

              AS.remove(A);
              features.put("rule", "more statistics whithout initial "+
                           "uppercase"+num+"than with (4.2.2)");
              AS.add(A.getStartNode(), A.getEndNode(),
                     "CapWordImpossible", features);

          // certain capitalized word
          } else if ( (up_u + up_a) > ( (ratio-(ratio*(range/100)))
                                        * ((no_up==0)?1:no_up) )
                      // not an ambiguous common capitalized word
                      && !(up_a > up_u && lexicon) ) {

              AS.remove(A);
              features.put("rule", "more statistics whith initial "+
                           "uppercase than without"+num+"(4.2.3)");
              AS.add(A.getStartNode(), A.getEndNode(),
                     "CapWordCertain", features);

          // possible or impossible capitalized word
          } else if (lexicon) {

                  // the word is common word
                  AS.remove(A);
                  features.put("rule", "common words lexicon (4.2.4)");
                  AS.add(A.getStartNode(), A.getEndNode(),
                         "CapWordImpossible", features);
          }
      }
      }

      if (docEditor != null) {
          docEditor.setTypeSelected(null, "Abbreviation", false);
          docEditor.setTypeSelected(null, "CapWordCertain", true);
          docEditor.setTypeSelected(null, "CapWordPossible", true);
          docEditor.setTypeSelected(null, "CapWordImpossible", true);
      }
  }

  /**
   * Step 05.1.
   *
   * Add features to certain/possible capitalized words as a new tag :
   * CapitalizedWord. Occurrence of word upper-initial, lowercased,
   * upper-initial in unambiguous position, that is to say not preceding a
   * dot or a beginning of line, length of word, case, previous and next
   * word Token.category and Token.string, confidence of being capitalized
   * word.
   */
  private void step05_1() {

      String fileName;
      String grammar;
      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.FeatureMap features;
      gate.Annotation A;
      int start, end;
      String s;
      Object[] array;

      // iterate on all capitalized words
      HashSet CapWordAS = new HashSet();
      CapWordAS.add("CapWordPossible");
      if (!automatic) {
          CapWordAS.add("CapWordCertain");
          CapWordAS.add("CapWordImpossible");
      }
      aAS = AS.get(CapWordAS);
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start+1, end);
          s = s.replaceAll("([\\r\\n ]|(\'.*$))", "");
          features = A.getFeatures();
          if (A.getType().equals("CapWordPossible")) {
              features.put("previous_class", "possible");
              features.put("class", "possible");
          } else if (A.getType().equals("CapWordCertain")) {
              features.put("previous_class", "certain");
              features.put("class", "certain");
          } else {
              features.put("previous_class", "impossible");
              features.put("class", "impossible");
          }
          features.put("length", new Integer(s.length()).toString());
          if (s.toUpperCase().equals(s)) {
              features.put("case", "up");
          } else if (s.substring(0,1).toUpperCase().equals(
                       s.substring(0,1))
                  && s.substring(1).toLowerCase().equals(
                       s.substring(1))) {
              features.put("case", "cap");
          } else {
              features.put("case", "mix");
          }
          if (s.matches(".*[0-9].*")) {
              features.put("contains_digit", "true");
          } else {
              features.put("contains_digit", "false");
          }
          AS.add(A.getStartNode(), A.getEndNode(), "CapitalizedWord",
                 features);
      }
      }

      // add previous and next word Token.category and token.string as
      // feature to CapitalizedWord tagged words

      try {
      if (transducer05_1 == null) {
      // create transducer
      fileName = "segmentation05_1.jape";
      grammar =
"Phase: Step_5_1\n"+
"Input: Token SpaceToken CapitalizedWord\n"+
"Options: control = appelt\n"+
"\n"+
"Rule: Step_5_1_1\n"+
"\n"+
"((({Token.kind == number}|{Token.kind == word})\n"+
"({Token.string == \".\"})?):previous_word)?\n"+
"({CapitalizedWord}):word\n"+
"(({SpaceToken}|{Token.string == \"_\"})\n"+
"({Token.kind == number}|{Token.kind == word}):next_word)?\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.AnnotationSet pAS = (gate.AnnotationSet)bindings.get(\"previous_word\");\n"+
"gate.AnnotationSet nAS = (gate.AnnotationSet)bindings.get(\"next_word\");\n"+
"gate.Annotation A = (gate.Annotation)AS.iterator().next();\n"+
"gate.FeatureMap features = A.getFeatures();\n"+
"String s;\n"+
"if (pAS != null) {\n"+
"gate.Annotation pA = (gate.Annotation)pAS.iterator().next();\n"+
"s = (String)pA.getFeatures().get(\"string\");\n"+
"features.put(\"previous_string\", s);\n";
if (POSTagging) {
grammar +=
"features.put(\"previous_category\", pA.getFeatures().get(\"category\"));\n";
}
grammar +=
"if (s.substring(0,1).toUpperCase().equals(s.substring(0,1))) {\n"+
"features.put(\"previous_uppercased_initial\", \"true\");\n"+
"} else {\n"+
"features.put(\"previous_uppercased_initial\", \"false\");\n"+
"}\n"+
"if (s.toUpperCase().equals(s)) {\n"+
"    features.put(\"previous_case\", \"up\");\n"+
"} else if (s.toLowerCase().equals(s)) {\n"+
"    features.put(\"previous_case\", \"low\");\n"+
"} else if (s.substring(0,1).toUpperCase().equals(\n"+
"             s.substring(0,1))\n"+
"        && s.substring(1).toLowerCase().equals(\n"+
"             s.substring(1))) {\n"+
"    features.put(\"previous_case\", \"cap\");\n"+
"} else {\n"+
"    features.put(\"previous_case\", \"mix\");\n"+
"}\n"+
"}\n"+
"if (nAS != null) {\n"+
"gate.Annotation nA = (gate.Annotation)nAS.iterator().next();\n"+
"s = (String)nA.getFeatures().get(\"string\");\n"+
"features.put(\"next_string\", s);\n";
if (POSTagging) {
grammar +=
"features.put(\"next_category\", nA.getFeatures().get(\"category\"));\n";
}
grammar +=
"if (s.substring(0,1).toUpperCase().equals(s.substring(0,1))) {\n"+
"features.put(\"next_uppercased_initial\", \"true\");\n"+
"} else {\n"+
"features.put(\"next_uppercased_initial\", \"false\");\n"+
"}\n"+
"if (s.toUpperCase().equals(s)) {\n"+
"    features.put(\"next_case\", \"up\");\n"+
"} else if (s.toLowerCase().equals(s)) {\n"+
"    features.put(\"next_case\", \"low\");\n"+
"} else if (s.substring(0,1).toUpperCase().equals(\n"+
"             s.substring(0,1))\n"+
"        && s.substring(1).toLowerCase().equals(\n"+
"             s.substring(1))) {\n"+
"    features.put(\"next_case\", \"cap\");\n"+
"} else {\n"+
"    features.put(\"next_case\", \"mix\");\n"+
"}\n"+
"}\n"+
"}\n";
      transducer05_1 = createTransducer(grammar, document, inputASName,
                                        inputASName, fileName, encoding);
      } else {
          transducer05_1.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer05_1.addProgressListener(pListener);
      transducer05_1.addStatusListener(sListener);
      transducer05_1.execute();
      transducer05_1.removeProgressListener(pListener);
      transducer05_1.removeStatusListener(sListener);

      } catch (ExecutionException e) {
          e.printStackTrace();
          return;
      }

      // iterate on all CapitalizedWord
      aAS = AS.get("CapitalizedWord");

      // add some features
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          features = A.getFeatures();
          s = (String)features.get("previous_string");
          if (s != null) {
          if (s.length() >= 6 && prefixes.containsKey(s.substring(0,6))) {
              features.put("previous_prefix", s.substring(0,6));
          } else if (s.length() >= 5
                     && prefixes.containsKey(s.substring(0,5))) {
              features.put("previous_prefix", s.substring(0,5));
          } else if (s.length() >= 4
                     && prefixes.containsKey(s.substring(0,4))) {
              features.put("previous_prefix", s.substring(0,4));
          } else if (s.length() >= 3
                     && prefixes.containsKey(s.substring(0,3))) {
              features.put("previous_prefix", s.substring(0,3));
          } else if (s.length() >= 2
                     && prefixes.containsKey(s.substring(0,2))) {
              features.put("previous_prefix", s.substring(0,2));
          } else if (s.length() >= 1
                     && prefixes.containsKey(s.substring(0,1))) {
              features.put("previous_prefix", s.substring(0,1));
          }
          if (s.length() >= 6
              && suffixes.containsKey(s.substring(s.length()-6))) {
              features.put("previous_suffix", s.substring(s.length()-6));
          } else if (s.length() >= 5
                     && suffixes.containsKey(s.substring(s.length()-5))) {
              features.put("previous_suffix", s.substring(s.length()-5));
          } else if (s.length() >= 4
                     && suffixes.containsKey(s.substring(s.length()-4))) {
              features.put("previous_suffix", s.substring(s.length()-4));
          } else if (s.length() >= 3
                     && suffixes.containsKey(s.substring(s.length()-3))) {
              features.put("previous_suffix", s.substring(s.length()-3));
          } else if (s.length() >= 2
                     && suffixes.containsKey(s.substring(s.length()-2))) {
              features.put("previous_suffix", s.substring(s.length()-2));
          } else if (s.length() >= 1
                     && suffixes.containsKey(s.substring(s.length()-1))) {
              features.put("previous_suffix", s.substring(s.length()-1));
          }
          }
          s = (String)features.get("next_string");
          if (s != null) {
          if (s.length() >= 6 && prefixes.containsKey(s.substring(0,6))) {
              features.put("next_prefix", s.substring(0,6));
          } else if (s.length() >= 5
                     && prefixes.containsKey(s.substring(0,5))) {
              features.put("next_prefix", s.substring(0,5));
          } else if (s.length() >= 4
                     && prefixes.containsKey(s.substring(0,4))) {
              features.put("next_prefix", s.substring(0,4));
          } else if (s.length() >= 3
                     && prefixes.containsKey(s.substring(0,3))) {
              features.put("next_prefix", s.substring(0,3));
          } else if (s.length() >= 2
                     && prefixes.containsKey(s.substring(0,2))) {
              features.put("next_prefix", s.substring(0,2));
          } else if (s.length() >= 1
                     && prefixes.containsKey(s.substring(0,1))) {
              features.put("next_prefix", s.substring(0,1));
          }
          if (s.length() >= 6
              && suffixes.containsKey(s.substring(s.length()-6))) {
              features.put("next_suffix", s.substring(s.length()-6));
          } else if (s.length() >= 5
                     && suffixes.containsKey(s.substring(s.length()-5))) {
              features.put("next_suffix", s.substring(s.length()-5));
          } else if (s.length() >= 4
                     && suffixes.containsKey(s.substring(s.length()-4))) {
              features.put("next_suffix", s.substring(s.length()-4));
          } else if (s.length() >= 3
                     && suffixes.containsKey(s.substring(s.length()-3))) {
              features.put("next_suffix", s.substring(s.length()-3));
          } else if (s.length() >= 2
                     && suffixes.containsKey(s.substring(s.length()-2))) {
              features.put("next_suffix", s.substring(s.length()-2));
          } else if (s.length() >= 1
                     && suffixes.containsKey(s.substring(s.length()-1))) {
              features.put("next_suffix", s.substring(s.length()-1));
          }
          }
      }
      }

      if (docEditor != null) {
          docEditor.setTypeSelected(null, "CapWordCertain", false);
          docEditor.setTypeSelected(null, "CapWordPossible", false);
          docEditor.setTypeSelected(null, "CapWordImpossible", false);
          docEditor.setTypeSelected(null, "CapitalizedWord", true);
      }
  }

  /**
   * Step 05.2.
   *
   * Learn a model on these CapitalizedWord tagged words.
   *
   * Train a weka tree machine learning on the features of capitalized words
   * to learn the feature class with values certain, impossible and
   * possible.
   *
   * @parameter features array that define if a feature is enable, value 1,
   * or disable, value 0
   */
  private Vector step05_2(int[] features) {

      String fileName;
      PrintWriter out;
      URL newConfigFileURL;
      String config;
      java.util.Iterator iterator;
      String prefixes_values, suffixes_values, tags_values;
      MLEngine engine;
      Vector message, line;

      prefixes_values = "";
      iterator = prefixes.keySet().iterator();
      while (iterator.hasNext()) {
          prefixes_values += "<VALUE>"+(String)iterator.next()+"</VALUE>\n";
      }

      suffixes_values = "";
      iterator = suffixes.keySet().iterator();
      while (iterator.hasNext()) {
          suffixes_values += "<VALUE>"+(String)iterator.next()+"</VALUE>\n";
      }

      tags_values = "";
      iterator = tags.keySet().iterator();
      while (iterator.hasNext()) {
          tags_values += "<VALUE>"+(String)iterator.next()+"</VALUE>\n";
      }

      // create ML configuration
      fileName = "ml_5_2.xml";
      config =
"<?xml version=\"1.0\" encoding=\""+encoding+"\"?>\n"+
"<ML-CONFIG>\n"+
"  <DATASET>\n"+
"  <INSTANCE-TYPE>CapitalizedWord</INSTANCE-TYPE>\n"+
"\n";
if (features[0] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.uppercased_initial_unambiguous(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>uppercased_initial_unambiguous</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[1] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.uppercased_initial_ambiguous(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>uppercased_initial_ambiguous</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[2] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.without_uppercased_initial(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>without_uppercased_initial</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[3] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.different_words_before(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>different_words_before</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[4] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.different_words_after(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>different_words_after</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[5] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.containing_expressions(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>containing_expressions</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[6] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.belongs_to_lexicon(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>belongs_to_lexicon</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[7] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.length(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>length</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[8] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.case(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>case</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>up</VALUE>\n"+
"      <VALUE>cap</VALUE>\n"+
"      <VALUE>mix</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[9] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.contains_digit(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>contains_digit</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[10] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.previous_category(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>previous_category</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
tags_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[11] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.previous_uppercased_initial(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>previous_uppercased_initial</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[12] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.previous_case(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>previous_case</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>low</VALUE>\n"+
"      <VALUE>up</VALUE>\n"+
"      <VALUE>cap</VALUE>\n"+
"      <VALUE>mix</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[13] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.previous_prefix(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>previous_prefix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
prefixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[14] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.previous_suffix(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>previous_suffix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
suffixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[15] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.next_category(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>next_category</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
tags_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[16] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.next_uppercased_initial(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>next_uppercased_initial</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>true</VALUE>\n"+
"      <VALUE>false</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[17] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.next_case(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>next_case</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>low</VALUE>\n"+
"      <VALUE>up</VALUE>\n"+
"      <VALUE>cap</VALUE>\n"+
"      <VALUE>mix</VALUE>\n"+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[18] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.next_prefix(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>next_prefix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
prefixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[19] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.next_suffix(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>next_suffix</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
suffixes_values+
"    </VALUES>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[20] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.user_cap_1(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>user_cap_1</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
if (features[21] == 1) { config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.user_cap_2(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>user_cap_2</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"  </ATTRIBUTE>\n"+
"\n";
}
config +=
"  <ATTRIBUTE>\n"+
"    <NAME>CapitalizedWord.class(0)</NAME>\n"+
"    <TYPE>CapitalizedWord</TYPE>\n"+
"    <FEATURE>class</FEATURE>\n"+
"    <POSITION>0</POSITION>\n"+
"    <VALUES>\n"+
"      <VALUE>certain</VALUE>\n"+
"      <VALUE>impossible</VALUE>\n"+
"      <VALUE>possible</VALUE>\n"+ 
"    </VALUES>\n"+
"    <CLASS/>\n"+
"  </ATTRIBUTE>\n"+
"\n"+
"  </DATASET>\n"+
"\n"+
"  <ENGINE>\n"+
"    <WRAPPER>gate.creole.ml.weka.Wrapper</WRAPPER>\n"+
"    <OPTIONS>\n"+
"        <CLASSIFIER OPTIONS=\"-M 4\">weka.classifiers.trees.J48</CLASSIFIER>\n"+
"    </OPTIONS>\n"+
"  </ENGINE>\n"+
"</ML-CONFIG>";
      try {
      newConfigFileURL = new URL(transducerDirectoryURL+"/"+fileName);
      out = new PrintWriter(new BufferedWriter(
           new FileWriter(new File(new URI(newConfigFileURL.toString())))));
      out.println(config);
      out.close();

      // create ML PR
      pnml = new MachineLearningPR();
      pnml.setConfigFileURL(newConfigFileURL);
      pnml.setDocument(document);
      pnml.init();

      // run ML PR in training mode
      pnml.setTraining(new Boolean(true));
      pnml.execute();

      // save data
      engine = pnml.getEngine();
      FileWriter fw = new FileWriter(
                  (new URL(transducerDirectoryURL+"/pnml.arff")).getPath());
      ((gate.creole.ml.weka.Wrapper)engine).saveDatasetAsARFF(fw);
      fw.close();

      // first apply a tree classifier
      message = new Vector();
      line = new Vector();
      Classifier c = new weka.classifiers.trees.J48();
      FileReader fr = new FileReader(
                  (new URL(transducerDirectoryURL+"/pnml.arff")).getPath());
      weka.core.Instances instances = new weka.core.Instances(fr);
      instances.setClassIndex(instances.numAttributes() - 1);
      // train classifier on all instances
      ((weka.classifiers.trees.J48)c).setSaveInstanceData(true);
      c.buildClassifier(instances);
      // test classifier on all instances
      EvaluationUtils ev = new EvaluationUtils();
      weka.core.FastVector fv = ev.getTestPredictions(c, instances);

      java.util.Enumeration predictions = fv.elements();
      weka.classifiers.evaluation.Prediction prediction;
      weka.core.Instances instances_unpredicted =
          new weka.core.Instances(instances, 0);
      message.add("C4.5 Unpredicted Instances");
      message.add("");
      Object[] annotations =
          document.getAnnotations().get("CapitalizedWord").toArray();
      java.util.Comparator comparator = new java.util.Comparator() {
              public int compare(Object o1, Object o2) {
                  return
                      (((SimpleAnnotation)o1).getStartNode().getOffset()
            .compareTo(((SimpleAnnotation)o2).getStartNode().getOffset()));
              }
          };
      // sort annotations the same way as predictions and instances
      java.util.Arrays.sort(annotations, comparator);
      int start, end;
      String rule;
      String word, context;
      String[] elements;
      gate.Annotation ann;
      Vector instances_unpredicted_context = new Vector();
      Vector instances_unpredicted_word = new Vector();
      for (int i = 0; predictions.hasMoreElements(); i++) {
          prediction = (weka.classifiers.evaluation.Prediction)
              predictions.nextElement();

          // if prediction failed for this instance
          if (prediction.actual() != prediction.predicted()) {

              // get text of this instance and context
              line = new Vector();
              ann = (Annotation)annotations[i];
              start = ann.getStartNode().getOffset().intValue();
              end = ann.getEndNode().getOffset().intValue();
              word = document.getContent().toString().substring(start, end);
              start -= 20; end += 20;
              if (start < 0) { start = 0; }
              if (end >= document.getContent().toString().length()) {
                  end = document.getContent().toString().length()-1; }
              context = document.getContent().toString()
                  .substring(start, end).replaceAll("\\p{Space}+", " ");
              line.add(word);
              instances_unpredicted_word.add(word);
              line.add("[...] "+context+" [...]");
              instances_unpredicted_context.add("[...] "+context+" [...]");
              // get features of this instance
              elements = instances.instance(i).toString().trim().split(",");
              for (int j = 0; j < elements.length; j++) {
                  line.add(elements[j]);
              }
              switch (new Double(prediction.predicted()).intValue()) {
              case 0 : line.add("certain"); break;
              case 1 : line.add("impossible"); break;
              case 2 : line.add("possible"); break;}
              rule = (ann.getFeatures().get("rule") != null)?
                  (String)ann.getFeatures().get("rule"):"";
              if (!ann.getFeatures().get("previous_class")
                  .equals(ann.getFeatures().get("class"))) {
                  line.add("user");
              } else if (prediction.actual() == 2) {
                  line.add("no process");
              } else {
                  line.add(rule);
              }
              message.add(line);
              instances_unpredicted.add(instances.instance(i));
          }
      }
      message.add("");
      message.add(instances_unpredicted.numInstances()+
                  " unpredicted on "+instances.numInstances()+
                  " ("+new java.text.DecimalFormat("#0.00")
                  .format(100.00*instances_unpredicted.numInstances()
                          /instances.numInstances())+"%)");

      // second apply a rule classifier on previous unpredicted instances
      message.add(""); message.add("");
      message.add("JRip Unpredicted Instances");
      message.add("");
      Classifier c2 = new weka.classifiers.rules.JRip();
      try {
          fv = ev.getTrainTestPredictions(c2, instances_unpredicted,
                                          instances_unpredicted);
      } catch (Exception e) {
          // not enough data to build the classifier
          message.add(e.getMessage());
          return message;
      }

      predictions = fv.elements();
      int num = 0;
      for (int i = 0; predictions.hasMoreElements(); i++) {
          prediction = (weka.classifiers.evaluation.Prediction)
              predictions.nextElement();

          // if prediction failed for this instance
          if (prediction.actual() != prediction.predicted()) {
              line = new Vector();
              line.add(instances_unpredicted_word.get(i));
              line.add(instances_unpredicted_context.get(i));
              // get features of this instance
              elements = instances_unpredicted.instance(i)
                  .toString().trim().split(",");
              for (int j = 0; j < elements.length; j++) {
                  line.add(elements[j]);
              }
              switch (new Double(prediction.predicted()).intValue()) {
              case 0 : line.add("certain"); break;
              case 1 : line.add("impossible"); break;
              case 2 : line.add("possible"); break;}
              message.add(line);
              num++;
          }
      }
      message.add("");
      message.add(num+" unpredicted on "
                  +instances_unpredicted.numInstances());

      // display tree classifier
      final javax.swing.JFrame jf = 
          new javax.swing.JFrame("Weka Classifier Tree Visualizer: J48");
      jf.setSize(500, 400);
      jf.getContentPane().setLayout(new java.awt.BorderLayout());
      weka.gui.treevisualizer.TreeVisualizer tv =
          new weka.gui.treevisualizer.TreeVisualizer(null,
                                    ((weka.classifiers.trees.J48)c).graph(),
                                  new weka.gui.treevisualizer.PlaceNode2());
      jf.getContentPane().add(tv, java.awt.BorderLayout.CENTER);
      jf.addWindowListener(new java.awt.event.WindowAdapter() {
              public void windowClosing(java.awt.event.WindowEvent e) {
                  jf.dispose();
              }
          });
      jf.setVisible(true);
      tv.fitToScreen();

      } catch (Exception e) {
          e.printStackTrace();
          return null;
      }

      return message;
  }

  /**
   * Step 6.
   *
   * Use statistics to tag possible capitalized words and possible
   * abbreviations as certain or impossible capitalized words and
   * abbreviations.
   *
   * @param min minimum number of occurrences in statistics needed to use
   * them
   * @param ratio ratio between occurrences for Certain and Impossible
   * capitalized words and abbreviations to class Possible abbreviations and
   * capitalized words in certain or impossible classes
   * @param wapWord true if capitalized words are processed, false if
   * abbrevations are processed
   */
    private void step06(double min, double ratio, boolean capWord) {

      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.Annotation A, A2;
      int start, end;
      String s, s2;
      Object[] array;
      java.util.HashMap countCertain;
      java.util.HashMap countImpossible;
      boolean  bound;
      double certain, impossible;
      FeatureMap features;

      // iterate on some annotations
      HashSet hSet = new HashSet();
      if (capWord) {
          hSet.add("CapWordPossible");
          hSet.add("AbbrCertain");
          hSet.add("AbbrImpossible");
      } else {
          hSet.add("AbbrPossible");
          hSet.add("CapWordCertain");
          hSet.add("CapWordImpossible");
      }
      aAS = AS.get(hSet);
      countCertain = new java.util.HashMap();
      countImpossible = new java.util.HashMap();

      if (aAS != null) {
      array = aAS.toArray();
      // sort annotations in the same order as text
      java.util.Comparator comparator = new java.util.Comparator() {
              public int compare(Object o1, Object o2) {
                  return
                      (((SimpleAnnotation)o1).getStartNode().getOffset()
            .compareTo(((SimpleAnnotation)o2).getStartNode().getOffset()));
              }
          };
      java.util.Arrays.sort(array, comparator);

      for (int i = 0; i < (array.length-1) ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end);
          A2 = (gate.Annotation)array[i+1];
          start = A2.getStartNode().getOffset().intValue();
          end = A2.getEndNode().getOffset().intValue();
          s2 = document.getContent().toString().substring(start, end);

          // test if the annotations are bound
          bound = (A.getEndNode().getOffset().intValue()
                   == A2.getStartNode().getOffset().intValue());
          if (!bound) { continue; }

          if (capWord && A2.getType().equals("CapWordPossible")) {
              if (A.getType().equals("AbbrCertain")) {
                  countCertain.put(s2, new Integer(
                                            (countCertain.get(s2) == null) ?
                         1:(((Integer)countCertain.get(s2)).intValue()+1)));
              } else if (A.getType().equals("AbbrImpossible")) {
                  countImpossible.put(s2, new Integer(
                                         (countImpossible.get(s2) == null) ?
                      1:(((Integer)countImpossible.get(s2)).intValue()+1)));
              }

          } else if (!capWord && A.getType().equals("AbbrPossible")) {
              if (A2.getType().equals("CapWordCertain")) {
                  countCertain.put(s, new Integer(
                                             (countCertain.get(s) == null) ?
                          1:(((Integer)countCertain.get(s)).intValue()+1)));
              } else if (A2.getType().equals("CapWordImpossible")) {
                  countImpossible.put(s, new Integer(
                                          (countImpossible.get(s) == null) ?
                       1:(((Integer)countImpossible.get(s)).intValue()+1)));
              }
          }
      }
      }

      // iterate on all CapWordPossible or AbbrPossible
      if (capWord) {
          aAS = AS.get("CapWordPossible");
      }  else {
          aAS = AS.get("AbbrPossible");
      }
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end);
          features = A.getFeatures();

          certain = (countCertain.get(s) == null) ?
              0:((Integer)countCertain.get(s)).intValue();
          impossible = (countImpossible.get(s) == null) ?
              0:((Integer)countImpossible.get(s)).intValue();
          features.put("Certain count", (new Double(certain)).toString());
          features.put("Impossible count",
                       (new Double(impossible)).toString());

          if ((certain + impossible) >= min) {
              if ((certain * ratio) >= impossible) {
                  if (capWord) {
                      AS.add(A.getStartNode(), A.getEndNode(),
                             "CapWordCertainNew", features);
                  } else {
                      AS.add(A.getStartNode(), A.getEndNode(),
                             "AbbrCertainNew", features);
                  }
                  AS.remove(A);

              } else if ((impossible * ratio) >= certain) {
                  if (capWord) {
                      AS.add(A.getStartNode(), A.getEndNode(),
                             "CapWordImpossibleNew", features);
                  } else {
                      AS.add(A.getStartNode(), A.getEndNode(),
                             "AbbrImpossibleNew", features);
                  }
                  AS.remove(A);
              }
          }
      }
      }

      if (docEditor != null) {
          docEditor.setTypeSelected(null, "CapitalizedWord", false);
          docEditor.setTypeSelected(null, "CapWordCertainNew", true);
          docEditor.setTypeSelected(null, "CapWordImpossibleNew", true);
          docEditor.setTypeSelected(null, "AbbrCertainNew", true);
          docEditor.setTypeSelected(null, "AbbrImpossibleNew", true);
      }
  }

  /**
   * Step 07.1.
   *
   * Apply a learned a model on Abbreviation tagged words of the full
   * corpus.
   *
   */
  private void step07_1() {

      PrintWriter out;
      URL path;
      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.Annotation A;
      int start, end;
      String s;
      Object[] array;

      try {
      abbml.setDocument(document);

      // run ML PR in applying mode
      abbml.setTraining(new Boolean(false));
      abbml.execute();

      // iterate on all AbbrCertain
      aAS = AS.get("AbbrCertain");
      if (aAS != null) {

      // save certain abbreviations list in a file
      path = new URL(resultsDirectoryURL+"/abbreviations.txt");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                new File(new URI(path.toString())), true)));

      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end-1);
          s = s.replaceAll("\'.*$", "");
          if (A.getFeatures().get("rule") != null
           && !A.getFeatures().get("rule")
              .equals("abbreviation lexicon rule (2.2.7)")) {
              out.println(s);
          }
      }

      out.close();
      }

      // iterate on all AbbrPossible
      aAS = AS.get("AbbrPossible");
      if (aAS != null) {

      // save possible abbreviations list in a file
      path = new URL(resultsDirectoryURL+"/abbreviations_possible.txt");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                new File(new URI(path.toString())), true)));

      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start, end-1);
          s = s.replaceAll("\'.*$", "");
          if (A.getFeatures().get("rule") != null
           && !A.getFeatures().get("rule")
              .equals("abbreviation lexicon rule (2.2.7)")) {
              out.println(s);
          }
      }

      out.close();
      }

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 07.2.
   *
   * Apply a learned a model on CapitalizedWord tagged words of the full
   * corpus.
   *
   */
  private void step07_2() {

      PrintWriter out;
      URL path;
      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.Annotation A;
      int start, end;
      String s;
      Object[] array;

      try {
      pnml.setDocument(document);

      // run ML PR in applying mode
      pnml.setTraining(new Boolean(false));
      pnml.execute();

      // iterate on all CapWordCertain
      aAS = AS.get("CapWordCertain");
      if (aAS != null) {

      // save certain capitalized words list in a file
      path = new URL(resultsDirectoryURL+"/capitalized_words.txt");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                new File(new URI(path.toString())), true)));

      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start+1, end);
          s = s.replaceAll("([\\r\\n ]|(\'.*$))", "");
          if (A.getFeatures().get("rule") != null
           && !A.getFeatures().get("rule")
              .equals("capitalized word lexicon (4.1.1)")) {
              out.println(s);
          }
      }

      out.close();
      }

      // iterate on all CapWordPossible
      aAS = AS.get("CapWordPossible");
      if (aAS != null) {

      // save possible capitalized words list in a file
      path = new URL(resultsDirectoryURL+"/capitalized_words_possible.txt");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                new File(new URI(path.toString())), true)));

      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          start = A.getStartNode().getOffset().intValue();
          end = A.getEndNode().getOffset().intValue();
          s = document.getContent().toString().substring(start+1, end);
          s = s.replaceAll("([\\r\\n ]|(\'.*$))", "");
          if (A.getFeatures().get("rule") != null
           && !A.getFeatures().get("rule")
              .equals("capitalized word lexicon (4.1.1)")) {
              out.println(s);
          }
      }

      out.close();
      }

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * Step 07.3.
   *
   * Tag the sentences boundaries and add end of line on SplitCertain tags.
   */
  private void step07_3() {

      AnnotationSet AS = document.getAnnotations();
      gate.AnnotationSet aAS;
      gate.Annotation A, A2;
      Iterator iterator;
      Object[] array;
      String fileName;
      String grammar;
      PrintWriter out;
      URL path;
      HashSet hAS;

      // iterate on all AbbrPossible
      aAS = AS.get("AbbrPossible");
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          // remove AbbrPossible annotation
          AS.remove(A);

          // get Abbreviation annotation for this expression
          iterator = AS.getContained(A.getStartNode().getOffset(),
                                     A.getEndNode().getOffset()).iterator();
          A2 = (gate.Annotation)iterator.next();
          while (iterator.hasNext()
                 && !(A2.getFeatures().containsKey("class")
                      && A2.getFeatures().size() == 1)) {
              A2 = (gate.Annotation)iterator.next();
          }

          // replace AbbrPossible tag according to the learned class
          if (A2.getFeatures().get("class").equals("impossible")) {
              AS.add(A.getStartNode(), A.getEndNode(),
                     "AbbrImpossible", A.getFeatures());
          } else if (A2.getFeatures().get("class").equals("certain")) {
              AS.add(A.getStartNode(), A.getEndNode(),
                     "AbbrCertain", A.getFeatures());
          } else {
              AS.add(A);
          }
      }
      }

      // iterate on all CapWordPossible
      aAS = AS.get("CapWordPossible");
      if (aAS != null) {
      array = aAS.toArray();
      for (int i = 0; i < array.length ; i++) {

          A = (gate.Annotation)array[i];
          // remove CapWordPossible annotation
          AS.remove(A);

          // get CapitalizedWord annotation for this expression
          iterator = AS.getContained(A.getStartNode().getOffset(),
                                     A.getEndNode().getOffset()).iterator();
          A2 = (gate.Annotation)iterator.next();
          while (iterator.hasNext()
                 && !(A2.getFeatures().containsKey("class")
                      && A2.getFeatures().size() == 1)) {
              A2 = (gate.Annotation)iterator.next();
          }

          // replace CapWordPossible tag according to the learned class
          if (A2.getFeatures().get("class").equals("impossible")) {
              AS.add(A.getStartNode(), A.getEndNode(),
                     "CapWordImpossible", A.getFeatures());
          } else if (A2.getFeatures().get("class").equals("certain")) {
              AS.add(A.getStartNode(), A.getEndNode(),
                     "CapWordCertain", A.getFeatures());
          } else {
              AS.add(A);
          }
      }
      }

      try {
      if (transducer07_3 == null) {
      // create transducer
      fileName = "segmentation07_3.jape";
/*
 SENTSTART : cap, maj, mix, [, (

// AbbrImpossible AbbrImpossible CapWordImpossible -> SplitCertain 1,3
// AbbrCertain    AbbrImpossible CapWordImpossible -> SplitCertain 2,3

 AbbrCertain    CapWordCertain       -> SplitImpossible
 AbbrCertain    CapWordImpossible    -> SplitCertain
 AbbrImpossible CapWordImpossible    -> SplitCertain
 AbbrImpossible CapWordCertain       -> SplitCertain

 )., ]., . *.   SENTSTART            -> SplitCertain
 ?,!            SENTSTART            -> SplitCertain
 ?,!,. puis ',",`          SENTSTART -> SplitCertain
 ?,!,.          ',",` puis SENTSTART -> SplitCertain
 ',",` puis ?,!,.          SENTSTART -> SplitCertain

 AbbrImpossible SENTSTART            -> SplitCertain
 AbbrCertain    min, num             -> SplitImpossible
 .              CapWordImpossible    -> SplitCertain

 .                                   -> SplitPossible
*/
      grammar =
"Phase: Step_7_3\n"+
"Input: Token SpaceToken AbbrCertain AbbrImpossible "+
"CapWordCertain CapWordImpossible "+lineAnnotationsToSkip+"\n"+
"Options: control = appelt\n"+
"\n"+
"Macro: SENTSTART\n"+
"({Token.orth == upperInitial}|{Token.orth == allCaps}\n"+
"|{Token.orth == mixedCaps}\n"+
"|{Token.string == \"[\"}|{Token.string == \"(\"})\n"+
"\n"+
"Rule: Step_7_3_0\n"+
"\n"+
"{SpaceToken}({"+lineAnnotationsToSkipRule+"}){SpaceToken}\n"+
"\n"+
"-->\n"+
"\n"+
"{}\n"+
"\n"+
// "Rule: Step_7_3_00\n"+
// "\n"+
// "((({AbbrImpossible}):word({SpaceToken.kind == space})+{AbbrImpossible}\n"+
// "({SpaceToken.kind == space})*{CapWordImpossible})\n"+
// "|({AbbrCertain}({SpaceToken.kind == space})+({AbbrImpossible}):word\n"+
// "({SpaceToken.kind == space})*{CapWordImpossible}))\n"+
// "\n"+
// "-->\n"+
// "\n"+
// "{\n"+
// "gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
// "gate.FeatureMap features = Factory.newFeatureMap();\n"+
// "features.put(\"rule\", \"7.3.00\");\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
// "}\n"+
"Rule: Step_7_3_1\n"+
"\n"+
"({AbbrCertain}):word"+
"({SpaceToken.kind == space})*{CapWordCertain}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.1\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitImpossible\", features);\n"+
"}\n"+
"Rule: Step_7_3_2\n"+
"\n"+
"({AbbrCertain}):word"+
"({SpaceToken.kind == space})*{CapWordImpossible}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.2\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_3\n"+
"\n"+
"({AbbrImpossible}):word"+
"({SpaceToken.kind == space})*({CapWordImpossible}|{CapWordCertain})\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.3\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_6\n"+
"\n"+
"({Token.string == \")\"}|{Token.string == \"]\"}"+
"|({Token.string == \".\"}({SpaceToken.kind == space})*))"+
"({Token.string == \".\"}):word({SpaceToken.kind == space})+\n"+
"SENTSTART\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.6\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_7\n"+
"\n"+
"({Token.string == \"?\"}|{Token.string == \"!\"}):word"+
"({SpaceToken.kind == space})+\n"+
"SENTSTART\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.7\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_8\n"+
"\n"+
"(({Token.string == \"?\"}|{Token.string == \"!\"}|\n"+
"{Token.string == \".\"})\n"+
"({Token.string == \"\'\"}|{Token.string == \"\\\"\"}|\n"+
"{Token.string == \"`\"})+):word({SpaceToken.kind == space})+\n"+
"SENTSTART\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.8\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_9\n"+
"\n"+
"({Token.string == \"?\"}|{Token.string == \"!\"}|\n"+
"{Token.string == \".\"}):word({SpaceToken.kind == space})+\n"+
"({Token.string == \"\'\"}|{Token.string == \"\\\"\"}|\n"+
"{Token.string == \"`\"})+\n"+
"SENTSTART\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.9\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_10\n"+
"\n"+
"(({Token.string == \"\'\"}|{Token.string == \"\\\"\"}|\n"+
"{Token.string == \"`\"})\n"+
"({Token.string == \"?\"}|{Token.string == \"!\"}|\n"+
"{Token.string == \".\"})+):word({SpaceToken.kind == space})+\n"+
"SENTSTART\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.10\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_4\n"+
"\n"+
"({AbbrImpossible}):word"+
"({SpaceToken.kind == space})*SENTSTART\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.4\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_5\n"+
"\n"+
"({AbbrCertain}):word"+
"({SpaceToken.kind == space})*\n"+
"({Token.orth == lowercase}|{Token.kind == number})\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.5\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitImpossible\", features);\n"+
"}\n"+
// "Rule: Step_7_3_11\n"+
// "\n"+
// "({AbbrImpossible}):word"+
// "({SpaceToken.kind == space})*{CapWordPossible}\n"+
// "\n"+
// "-->\n"+
// "\n"+
// "{\n"+
// "gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
// "gate.FeatureMap features = Factory.newFeatureMap();\n"+
// "features.put(\"rule\", \"7.3.11\");\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
// "}\n"+
// "Rule: Step_7_3_12\n"+
// "\n"+
// "({AbbrPossible}):word"+
// "({SpaceToken.kind == space})*{CapWordImpossible}\n"+
// "\n"+
// "-->\n"+
// "\n"+
// "{\n"+
// "gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
// "gate.FeatureMap features = Factory.newFeatureMap();\n"+
// "features.put(\"rule\", \"7.3.12\");\n"+
// "outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
// "}\n"+
"Rule: Step_7_3_13\n"+
"\n"+
"({Token.string == \".\"}):word"+
"({SpaceToken.kind == space})*{CapWordImpossible}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.13\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitCertain\", features);\n"+
"}\n"+
"Rule: Step_7_3_14\n"+
"\n"+
"({Token.string == \".\"}):word"+
"{SpaceToken.kind == space}\n"+
"\n"+
"-->\n"+
"\n"+
"{\n"+
"gate.AnnotationSet AS = (gate.AnnotationSet)bindings.get(\"word\");\n"+
"gate.FeatureMap features = Factory.newFeatureMap();\n"+
"features.put(\"rule\", \"7.3.14\");\n"+
"outputAS.add(AS.firstNode(), AS.lastNode(), \"SplitPossible\", features);\n"+
"}\n";
      transducer07_3 = createTransducer(grammar, document, inputASName,
                                        inputASName, fileName, encoding);
      } else {
          transducer07_3.setDocument(document);
      }

      // run transducer
      fireStatusChanged("Run the transducer");
      if (isInterrupted()) {
          throw new ExecutionInterruptedException("The execution of the \""+
          getName()+"\" sentence splitter has been abruptly interrupted!");
      }
      pListener = new IntervalProgressListener(0, 100);
      transducer07_3.addProgressListener(pListener);
      transducer07_3.addStatusListener(sListener);
      transducer07_3.execute();
      transducer07_3.removeProgressListener(pListener);
      transducer07_3.removeStatusListener(sListener);

      // save input file in new file
      fileName = (new File(new URI(document.getSourceUrl().toString())))
          .getName();
      path = new URL(resultsDirectoryURL+"/"+fileName+".split");
      out = new PrintWriter(new BufferedWriter(new FileWriter(
                                new File(new URI(path.toString())))));

      // annotations to keep in the XML output
      hAS = new HashSet();
      hAS.add("SplitCertain");
      hAS.add("SplitPossible");
      hAS.addAll(document.getAnnotations("Original markups").getAllTypes());
      hAS.remove("body"); hAS.remove("BODY");
      hAS.remove("head"); hAS.remove("HEAD");
      hAS.remove("html"); hAS.remove("HTML");
      hAS.remove("meta"); hAS.remove("META");

      // remove annotations from "Original markups" AnnotationSet
      document.getAnnotations("Original markups").clear();

      // write XML output
      String text, newtext;
      text = document.toXml(AS.get(hAS), false);
      text = text.replaceAll("<\\?xml version=[^>]+>", "");
      text = text.replaceAll("<>", "&lt;&gt;");
      text = text.replaceAll("((^|>| )[^<> ]*)>", "$1&gt;");
      text = text.replaceAll("<([^<> ]*( |<|$))", "&lt;$1");

      // split a sentence per line on SplitCertain tags
      // if SplitCertain tags are contained into other tags
      newtext = text.replaceAll(
                  "\n<([^ >]+)>(.+?)<SplitCertain>([^<]+)</SplitCertain> ?",
                             "\n<$1>$2$3</$1>\n<$1>");
      while (text.length() != newtext.length() && !text.equals(newtext)) {
          text = newtext;
          newtext = text.replaceAll(
                  "\n<([^ >]+)>(.+?)<SplitCertain>([^<]+)</SplitCertain> ?",
                             "\n<$1>$2$3</$1>\n<$1>");
      }
      // if SplitCertain tags are not contained into other tags
      text = text.replaceAll("<SplitCertain>([^<]+)</SplitCertain>",
                             "$1\n");

      // add XML header and footer around text
      text =
"<?xml version=\"1.0\" encoding=\""+encoding+"\"?>\n"+
"<!DOCTYPE html\n"+
"	PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"+
"	\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"+
"<html>\n"+
"<head>\n"+
"<meta http-equiv=\"Content-Type\" content=\"text/html; charset="+encoding+"\" />\n"+
"</head>\n"+
"<body>\n"+text+"\n"+
"</body>\n"+
"</html>";

      out.println(text);
      out.close();

      } catch (Exception e) {
          e.printStackTrace();
          return;
      }
  }

  /**
   * End.
   */
  private void end() {

      // get pointers to the annotation sets
      AnnotationSet inputAS = (inputASName == null) ?
          document.getAnnotations() :
          document.getAnnotations(inputASName);

      AnnotationSet outputAS = (outputASName == null) ?
          document.getAnnotations() :
          document.getAnnotations(outputASName);

      // copy the results to the output set if they are different
      if (inputAS != outputAS) {
          outputAS.addAll(inputAS.get(SENTENCE_ANNOTATION_TYPE));
      }

      // create one big sentence if none were found
      AnnotationSet sentences = outputAS.get(SENTENCE_ANNOTATION_TYPE);
      if (sentences == null || sentences.isEmpty()) {
          //create an annotation covering the entire content
          try {
          outputAS.add(new Long(0), document.getContent().size(), 
                       SENTENCE_ANNOTATION_TYPE, Factory.newFeatureMap());
          } catch (InvalidOffsetException e) {
              e.printStackTrace();
              return;
          }

      } else {
          // add a sentence covering all the tokens after the last sentence
          Long endSentences = sentences.lastNode().getOffset();
          AnnotationSet remainingTokens =
              inputAS.get(TOKEN_ANNOTATION_TYPE, endSentences,
                          inputAS.lastNode().getOffset());
          if (remainingTokens != null && !remainingTokens.isEmpty()) {
              try {
              outputAS.add(remainingTokens.firstNode().getOffset(),
                           remainingTokens.lastNode().getOffset(),
                           SENTENCE_ANNOTATION_TYPE,
                           Factory.newFeatureMap());
              } catch(InvalidOffsetException e) {
                  e.printStackTrace();
                  return;
              }
          }
      }

      fireProcessFinished();
  }

  /**
   * Create dynamically a transducer. Write a JAPE transducer in a file then
   * load it with Factory.createResource(...).
   *
   * @param grammar string of the JAPE grammar, null if the file already
   * exist
   * @return a transducer
   */
  private Transducer createTransducer(String grammar,
                                      gate.Document document,
                                      String inputASName,
                                      String outputASName,
                                      String fileName, String encoding)
      throws ExecutionException {

      Transducer transducer;
      FeatureMap params;
      PrintWriter out;
      URL newTransducerDirectoryURL;

      if (inputASName != null && inputASName.equals("")) {
          inputASName = null;
      }
      if (outputASName != null && outputASName.equals("")) {
          outputASName = null;
      }
      try {
          newTransducerDirectoryURL =
              new URL(transducerDirectoryURL+"/"+fileName);
      } catch (MalformedURLException e) {
          throw new ExecutionException(e);
      }

      // write a Jape grammar file
      if (grammar != null) {
      try {
      out = new PrintWriter(new BufferedWriter(new FileWriter(new File(
                          new URI(newTransducerDirectoryURL.toString())))));
      out.println(grammar);
      out.close();

      } catch (Exception e) {
          throw new ExecutionException(e);
      }
      }

      // set hidden parameters of the transducer
      params = Factory.newFeatureMap();
      if (newTransducerDirectoryURL != null) {
          params.put(Transducer.TRANSD_GRAMMAR_URL_PARAMETER_NAME,
                     newTransducerDirectoryURL);
      }
      params.put(Transducer.TRANSD_ENCODING_PARAMETER_NAME, encoding);
      features = Factory.newFeatureMap();
      Gate.setHiddenAttribute(features, true);

      // create the transducer
      try {
      transducer = (Transducer)Factory.createResource(
                                "gate.creole.Transducer", params, features);
      transducer.setName("Transducer " + System.currentTimeMillis());

      // set other parameters of the transducer
      params = Factory.newFeatureMap();
      params.put(Transducer.TRANSD_DOCUMENT_PARAMETER_NAME, document);
      params.put(Transducer.TRANSD_INPUT_AS_PARAMETER_NAME, inputASName);
      params.put(Transducer.TRANSD_OUTPUT_AS_PARAMETER_NAME, inputASName);
      transducer.setParameterValues(params);

      } catch (Exception e) {
          throw new ExecutionException(e);
      }

      return transducer;
  }

  /**
   * Let the user choose parameters to class word into 3 classes : certain,
   * possible and impossible abbreviations.
   */
  public class StatisticsDialog extends javax.swing.JDialog {
    
      /** Creates new form dialog */
      public StatisticsDialog(java.awt.Frame parent, boolean modal,
                              int[] parameters) {

          super(parent, modal);
          initComponents();
          jSlider1.setValue(parameters[0]);
          jSlider2.setValue(parameters[1]);
          jSlider3.setValue(parameters[2]);
          this.getRootPane().setDefaultButton(jButton2);
      }
    
      /** This method is called from within the constructor to
       * initialize the form.
       */
      private void initComponents() {
          jLabel4 = new javax.swing.JLabel();
          jPanel1 = new javax.swing.JPanel();
          jLabel1 = new javax.swing.JLabel();
          jSlider1 = new javax.swing.JSlider();
          jLabel2 = new javax.swing.JLabel();
          jSlider2 = new javax.swing.JSlider();
          jLabel3 = new javax.swing.JLabel();
          jSlider3 = new javax.swing.JSlider();
          jPanel2 = new javax.swing.JPanel();
          jButton1 = new javax.swing.JButton();
          jButton2 = new javax.swing.JButton();

          getContentPane().setLayout(new java.awt.BorderLayout(20, 20));

          setDefaultCloseOperation(
              javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
          setTitle("Step 2/7 - Classify abbreviations");
          setBackground(new java.awt.Color(204, 204, 204));
          setLocationRelativeTo(null);

          jLabel4.setText(
            "<html>Please verify that AbbrCertain, AbbrImpossible and"+
            "<br>AbbrPossible tags are correct.\n"+
            "<br>Modify tags if necessary. To do it move cursor on a tag"+
            "<br>and change the first field to one of the three tags.<br>"+
            "<br>An abbreviation is here a word always ending with a dot"+
            "<br>that is a shortcut for a longer expression."+
            "<br>An ambiguous abbreviation is an abbreviation preceding a"+
            "<br>capitalized word or is an abbreviation at end of line."+
            "</html>");
          getContentPane().add(jLabel4, java.awt.BorderLayout.NORTH);

          jPanel1.setLayout(new java.awt.GridLayout(6, 1, 10, 10));

          jLabel1.setText(
            "<html>Minimum number of total occurrences in statistics"+
            "<br>needed to use them."+
            "<br>Total occurrences = ending_with_dot_ambiguous"+
            "<br>ending_with_dot_unambiguous + not_ending_with_dot."+
            "</html>");
          jPanel1.add(jLabel1);

          jSlider1.setMinimum(0);
          jSlider1.setMaximum(50);
          jSlider1.setValue(2);
          jSlider1.setPaintLabels(true);
          jSlider1.setPaintTicks(true);
          jSlider1.setSnapToTicks(true);
          jSlider1.setMinorTickSpacing(1);
          jSlider1.setMajorTickSpacing(5);
          jPanel1.add(jSlider1);

          jLabel2.setText(
            "<html>Ratio between occurrences with dot and without dot to"+
            "<br>class abbreviations into certain or impossible classes."+
            "<br>Increase to augment impossible abbreviations and"+
            "<br>decrement certain abbreviations and conversely.</html>");
          jPanel1.add(jLabel2);

          jSlider2.setMinimum(1);
          jSlider2.setMaximum(10);
          jSlider2.setValue(3);
          jSlider2.setPaintLabels(true);
          jSlider2.setPaintTicks(true);
          jSlider2.setSnapToTicks(true);
          jSlider2.setMinorTickSpacing(1);
          jSlider2.setMajorTickSpacing(1);
          jPanel1.add(jSlider2);

          jLabel3.setText(
            "<html>Interval in percentage of the window of possible"+
            "<br>abbreviation knowing that below it is certain and up it"+
            "<br>is impossible class. Increase to augment possible"+
            "<br>abbreviations except common words that are classified as"+
            "<br>impossible.</html>");
          jPanel1.add(jLabel3);

          jSlider3.setMinimum(0);
          jSlider3.setMaximum(100);
          jSlider3.setValue(20);
          jSlider3.setPaintLabels(true);
          jSlider3.setPaintTicks(true);
          jSlider3.setSnapToTicks(true);
          jSlider3.setMinorTickSpacing(1);
          jSlider3.setMajorTickSpacing(10);
          jPanel1.add(jSlider3);

          getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

          jPanel2.setLayout(new java.awt.FlowLayout(
                            java.awt.FlowLayout.CENTER, 10, 10));

          jButton1.setText("Do it again");
          jButton1.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {

                      AnnotationSet AS = document.getAnnotations();
                      gate.AnnotationSet aAS;
                      gate.Annotation A;
                      Object[] array;

                      // iterate on all AbbrCertain and AbbrImpossible
                      HashSet AbbrAS = new HashSet();
                      AbbrAS.add("AbbrCertain");
                      AbbrAS.add("AbbrImpossible");
                      aAS = AS.get(AbbrAS);
                      if (aAS != null) {
                      array = aAS.toArray();
                      for (int i = 0; i < array.length ; i++) {

                          A = (gate.Annotation)array[i];
                          if (A.getFeatures().containsKey("rule")
                              && ((String)A.getFeatures().get("rule"))
                              .matches(".*\\(2\\.4\\..\\)")) {

                              // set back tags to precedent state
                              AS.remove(A);
                              AS.add(A.getStartNode(), A.getEndNode(),
                                     "AbbrPossible", A.getFeatures());
                          }
                      }
                      }

                      step02_4(
                             new Double(jSlider1.getValue()).doubleValue(),
                             new Double(jSlider2.getValue()).doubleValue(),
                             new Double(jSlider3.getValue()).doubleValue());
                  }
              });

          jPanel2.add(jButton1);

          jButton2.setText("Next Step");
          jButton2.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {
                      statisticsValues[0] =
                          new Double(jSlider1.getValue()).doubleValue();
                      statisticsValues[1] =
                          new Double(jSlider2.getValue()).doubleValue();
                      statisticsValues[2] =
                          new Double(jSlider3.getValue()).doubleValue();
                      setVisible(false);
                      dialog3.setVisible(true);
                  }
              });

          jPanel2.add(jButton2);

          getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

          pack();
          setLocation(getOwner().getWidth()/2-getWidth()/2, 0);
      }
   
      private javax.swing.JButton jButton1;
      private javax.swing.JButton jButton2;
      private javax.swing.JLabel jLabel1;
      private javax.swing.JLabel jLabel2;
      private javax.swing.JLabel jLabel3;
      private javax.swing.JPanel jPanel1;
      private javax.swing.JPanel jPanel2;
      private javax.swing.JSlider jSlider1;
      private javax.swing.JSlider jSlider2;
      private javax.swing.JSlider jSlider3;
      private javax.swing.JLabel jLabel4;
  }

  /**
   * Let the user choose parameters to class word into 3 classes : certain,
   * possible and impossible capitalized words.
   */
  public class Statistics2Dialog extends javax.swing.JDialog {
    
      /** Creates new form dialog */
      public Statistics2Dialog(java.awt.Frame parent, boolean modal,
                              int[] parameters) {

          super(parent, modal);
          initComponents();
          jSlider1.setValue(parameters[0]);
          jSlider2.setValue(parameters[1]);
          jSlider3.setValue(parameters[2]);
          this.getRootPane().setDefaultButton(jButton2);
      }
    
      /** This method is called from within the constructor to
       * initialize the form.
       */
      private void initComponents() {
          jLabel4 = new javax.swing.JLabel();
          jPanel1 = new javax.swing.JPanel();
          jLabel1 = new javax.swing.JLabel();
          jSlider1 = new javax.swing.JSlider();
          jLabel2 = new javax.swing.JLabel();
          jSlider2 = new javax.swing.JSlider();
          jLabel3 = new javax.swing.JLabel();
          jSlider3 = new javax.swing.JSlider();
          jPanel2 = new javax.swing.JPanel();
          jButton1 = new javax.swing.JButton();
          jButton2 = new javax.swing.JButton();

          getContentPane().setLayout(new java.awt.BorderLayout(20, 20));

          setDefaultCloseOperation(
              javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
          setTitle("Step 4/7 - Classify capitalized words");
          setBackground(new java.awt.Color(204, 204, 204));
          setLocationRelativeTo(null);

          jLabel4.setText(
            "<html>Please verify that CapWordCertain, CapWordImpossible"+
            "<br>and CapWordPossible are well tagged.\n"+
            "<br>Modify tags if necessary. To do it move cursor on a tag"+
            "<br>and change the first field to one of the three tags.<br>"+
            "<br>A capitalized word is here a word that is always"+
            "<br>beginning with an uppercased letter and that is not a "+
            "common word."+
            "<br>An ambiguous capitalized word is a word following a"+
            "<br>word ending with a dot or is a word starting a line."+
            "</html>"
            );
          getContentPane().add(jLabel4, java.awt.BorderLayout.NORTH);

          jPanel1.setLayout(new java.awt.GridLayout(6, 1, 10, 10));

          jLabel1.setText(
            "<html>Minimum number of total occurrences in statistics"+
            "<br>needed to use them."+
            "<br>Total occurrences = uppercased_initial_ambiguous"+
            "<br>uppercased_initial_unambiguous + "+
            "without_uppercased_initial."+
            "</html>");
          jPanel1.add(jLabel1);

          jSlider1.setMinimum(0);
          jSlider1.setMaximum(50);
          jSlider1.setValue(2);
          jSlider1.setPaintLabels(true);
          jSlider1.setPaintTicks(true);
          jSlider1.setSnapToTicks(true);
          jSlider1.setMinorTickSpacing(1);
          jSlider1.setMajorTickSpacing(5);
          jPanel1.add(jSlider1);

          jLabel2.setText(
            "<html>Ratio between occurrences with uppercased initial and"+
            "<br>without to class capitalized words into certain or"+
            "<br>impossible classes. Increase to augment impossible"+
            "<br>capitalized words and decrement certain capitalized words"+
            "<br>and conversely.</html>");
          jPanel1.add(jLabel2);

          jSlider2.setMinimum(1);
          jSlider2.setMaximum(10);
          jSlider2.setValue(3);
          jSlider2.setPaintLabels(true);
          jSlider2.setPaintTicks(true);
          jSlider2.setSnapToTicks(true);
          jSlider2.setMinorTickSpacing(1);
          jSlider2.setMajorTickSpacing(1);
          jPanel1.add(jSlider2);

          jLabel3.setText(
            "<html>Interval in percentage of the window of possible"+
            "<br>capitalized word knowing that below it is certain and up"+
            "<br>it is impossible class. Increase to augment possible"+
            "<br>capitalized words except common words that are classified"+
            "<br>as impossible.</html>");
          jPanel1.add(jLabel3);

          jSlider3.setMinimum(0);
          jSlider3.setMaximum(100);
          jSlider3.setValue(20);
          jSlider3.setPaintLabels(true);
          jSlider3.setPaintTicks(true);
          jSlider3.setSnapToTicks(true);
          jSlider3.setMinorTickSpacing(1);
          jSlider3.setMajorTickSpacing(10);
          jPanel1.add(jSlider3);

          getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

          jPanel2.setLayout(new java.awt.FlowLayout(
                            java.awt.FlowLayout.CENTER, 10, 10));

          jButton1.setText("Do it again");
          jButton1.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {

                      AnnotationSet AS = document.getAnnotations();
                      gate.AnnotationSet aAS;
                      gate.Annotation A;
                      Object[] array;

                      // iterate on all CapWordCertain and CapWordImpossible
                      HashSet CapWordAS = new HashSet();
                      CapWordAS.add("CapWordCertain");
                      CapWordAS.add("CapWordImpossible");
                      aAS = AS.get(CapWordAS);
                      if (aAS != null) {
                      array = aAS.toArray();
                      for (int i = 0; i < array.length ; i++) {

                          A = (gate.Annotation)array[i];
                          if (A.getFeatures().containsKey("rule")
                              && ((String)A.getFeatures().get("rule"))
                              .matches(".*\\(4\\.2\\..\\)")) {

                              // set back tags to precedent state
                              AS.remove(A);
                              AS.add(A.getStartNode(), A.getEndNode(),
                                     "CapWordPossible", A.getFeatures());
                          }
                      }
                      }

                      step04_2(
                             new Double(jSlider1.getValue()).doubleValue(),
                             new Double(jSlider2.getValue()).doubleValue(),
                             new Double(jSlider3.getValue()).doubleValue());
                  }
              });

          jPanel2.add(jButton1);

          jButton2.setText("Next Step");
          jButton2.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {
                      statisticsValues[3] =
                          new Double(jSlider1.getValue()).doubleValue();
                      statisticsValues[4] =
                          new Double(jSlider2.getValue()).doubleValue();
                      statisticsValues[5] =
                          new Double(jSlider3.getValue()).doubleValue();
                      setVisible(false);
                      dialog5.setVisible(true);
                  }
              });

          jPanel2.add(jButton2);

          getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

          pack();
          setLocation(getOwner().getWidth()/2-getWidth()/2, 0);
      }
   
      private javax.swing.JButton jButton1;
      private javax.swing.JButton jButton2;
      private javax.swing.JLabel jLabel1;
      private javax.swing.JLabel jLabel2;
      private javax.swing.JLabel jLabel3;
      private javax.swing.JPanel jPanel1;
      private javax.swing.JPanel jPanel2;
      private javax.swing.JSlider jSlider1;
      private javax.swing.JSlider jSlider2;
      private javax.swing.JSlider jSlider3;
      private javax.swing.JLabel jLabel4;
  }

  /**
   * Show the user the unpredicted instances with several learning
   * algorithms. Allow the user to choose features used to learn.
   */
  public class LearnAbbreviationsDialog extends javax.swing.JDialog {

      Vector columnId;

      /** Creates new form dialog */
      public LearnAbbreviationsDialog(java.awt.Frame parent,
                                      boolean modal, Vector message,
                                      int[] features) {
          super(parent, modal);
          initComponents();
          reinit(message, features);

          jCheckBox1.setSelected((features[0]==1)?true:false);
          jCheckBox2.setSelected((features[1]==1)?true:false);
          jCheckBox3.setSelected((features[2]==1)?true:false);
          jCheckBox4.setSelected((features[3]==1)?true:false);
          jCheckBox5.setSelected((features[4]==1)?true:false);
          jCheckBox6.setSelected((features[5]==1)?true:false);
          jCheckBox7.setSelected((features[6]==1)?true:false);
          jCheckBox8.setSelected((features[7]==1)?true:false);
          jCheckBox9.setSelected((features[8]==1)?true:false);
          jCheckBox10.setSelected((features[9]==1)?true:false);
          jCheckBox11.setSelected((features[10]==1)?true:false);
          jCheckBox12.setSelected((features[11]==1)?true:false);
          jCheckBox13.setSelected((features[12]==1)?true:false);
          jCheckBox14.setSelected((features[13]==1)?true:false);
          jCheckBox15.setSelected((features[14]==1)?true:false);
          jCheckBox16.setSelected((features[15]==1)?true:false);
          jCheckBox17.setSelected((features[16]==1)?true:false);
          jCheckBox18.setSelected((features[17]==1)?true:false);
          jCheckBox19.setSelected((features[18]==1)?true:false);
          jCheckBox20.setSelected((features[19]==1)?true:false);
          jCheckBox21.setSelected((features[20]==1)?true:false);

          getRootPane().setDefaultButton(jButton2);
          jButton2.requestFocus();
      }

      /** This method is called from within the constructor to
       * initialize the form.
       */
      private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jCheckBox2 = new javax.swing.JCheckBox();
        jCheckBox3 = new javax.swing.JCheckBox();
        jCheckBox4 = new javax.swing.JCheckBox();
        jCheckBox5 = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        jCheckBox6 = new javax.swing.JCheckBox();
        jCheckBox7 = new javax.swing.JCheckBox();
        jCheckBox8 = new javax.swing.JCheckBox();
        jLabel4 = new javax.swing.JLabel();
        jCheckBox9 = new javax.swing.JCheckBox();
        jCheckBox10 = new javax.swing.JCheckBox();
        jCheckBox11 = new javax.swing.JCheckBox();
        jCheckBox12 = new javax.swing.JCheckBox();
        jCheckBox13 = new javax.swing.JCheckBox();
        jLabel5 = new javax.swing.JLabel();
        jCheckBox14 = new javax.swing.JCheckBox();
        jCheckBox15 = new javax.swing.JCheckBox();
        jCheckBox16 = new javax.swing.JCheckBox();
        jCheckBox17 = new javax.swing.JCheckBox();
        jCheckBox18 = new javax.swing.JCheckBox();
        jCheckBox19 = new javax.swing.JCheckBox();
        jCheckBox20 = new javax.swing.JCheckBox();
        jCheckBox21 = new javax.swing.JCheckBox();

        setTitle("Step 3/7 - Classify abbreviations");
        getContentPane().setLayout(new java.awt.BorderLayout(10, 10));
        setDefaultCloseOperation(javax.swing.WindowConstants
                                 .DISPOSE_ON_CLOSE);

        jLabel1.setText(
          "<html>Please choose the features you want to use for learning. "+
          "Add or remove a feature by ticking check boxes on the "+
          "right pane."+
          "<br>You can see the unpredicted instances in the table. "+
          "First with C4.5 learning algorithm then JRip on precedent "+
          "C4.5 unpredicted instances."+
          "<br>It should help you to determine which feature to keep, "+
          "remove or add."+
          "<br>Processes that determine the value of a word is, in that "+
          "order: anti-abbr. dict., abbr. dict., syntactic rules, "+
          "statistics, common words dict., learned model."+
          "<br>You can change the <b>class</b> feature of annotations "+
          "<b>Abbreviation</b> directly in the text but it has sense only "+
          "on a line with a possible actual value.");
        getContentPane().add(jLabel1, java.awt.BorderLayout.NORTH);

        jButton1.setText("Do it again");
        jPanel2.add(jButton1);
        jButton2.setText("Next Step");
        jPanel2.add(jButton2);
        getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

        jSplitPane1.setResizeWeight(1);
        jSplitPane1.setContinuousLayout(true);

        jScrollPane1.setAutoscrolls(true);
        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jTable1.setAutoResizeMode(javax.swing.JTable
                                  .AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        jTable1.setCellSelectionEnabled(true);
        jScrollPane1.setViewportView(jTable1);
        jSplitPane1.setLeftComponent(jScrollPane1);

        jPanel1.setLayout(new java.awt.GridLayout(26, 2, 5, 5));
        jPanel1.setBorder(new javax.swing.border.EtchedBorder());

        jLabel2.setText("Global features on possible abbreviation");
        jPanel1.add(jLabel2);

        jCheckBox1.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox1.setText("ending with dot in unambiguous position");
        jPanel1.add(jCheckBox1);

        jCheckBox2.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox2.setText("ending with dot in ambiguous position");
        jPanel1.add(jCheckBox2);

        jCheckBox3.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox3.setText("not ending with dot");
        jPanel1.add(jCheckBox3);

        jCheckBox4.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox4.setText("different words before");
        jPanel1.add(jCheckBox4);

        jCheckBox5.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox5.setText("different words after");
        jPanel1.add(jCheckBox5);

        jCheckBox6.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox6.setText("belongs to lexicon");
        jPanel1.add(jCheckBox6);

        jLabel3.setText("Local features on possible abbreviation");
        jPanel1.add(jLabel3);

        jCheckBox7.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox7.setText("length");
        jPanel1.add(jCheckBox7);

        jCheckBox8.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox8.setText("case");
        jPanel1.add(jCheckBox8);

        jCheckBox9.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox9.setText("contains digit");
        jPanel1.add(jCheckBox9);

        jLabel4.setText("Local features on previous word");
        jPanel1.add(jLabel4);

        jCheckBox10.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox10.setText("previous word grammatical category");
        if (!POSTagging) { jCheckBox10.setEnabled(false); }
        jPanel1.add(jCheckBox10);

        jCheckBox11.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox11.setText("previous word ends with a dot");
        jPanel1.add(jCheckBox11);

        jCheckBox12.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox12.setText("previous word case");
        jPanel1.add(jCheckBox12);

        jCheckBox13.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox13.setText("previous word prefix");
        jPanel1.add(jCheckBox13);

        jCheckBox14.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox14.setText("previous word suffix");
        jPanel1.add(jCheckBox14);

        jLabel5.setText("Local features on next word");
        jPanel1.add(jLabel5);

        jCheckBox15.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox15.setText("next word grammatical category");
        if (!POSTagging) { jCheckBox15.setEnabled(false); }
        jPanel1.add(jCheckBox15);

        jCheckBox16.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox16.setText("next word ends with a dot");
        jPanel1.add(jCheckBox16);

        jCheckBox17.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox17.setText("next word case");
        jPanel1.add(jCheckBox17);

        jCheckBox18.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox18.setText("next word prefix");
        jPanel1.add(jCheckBox18);

        jCheckBox19.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox19.setText("next word suffix");
        jPanel1.add(jCheckBox19);

        jCheckBox20.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox20.setText("user define feature 1");
        jPanel1.add(jCheckBox20);

        jCheckBox21.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox21.setText("user define feature 2");
        jPanel1.add(jCheckBox21);

        jSplitPane1.setRightComponent(jPanel1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        pack();

        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                int[] a = {
                    jCheckBox1.isSelected()?1:0,
                    jCheckBox2.isSelected()?1:0,
                    jCheckBox3.isSelected()?1:0,
                    jCheckBox4.isSelected()?1:0,
                    jCheckBox5.isSelected()?1:0,
                    jCheckBox6.isSelected()?1:0,
                    jCheckBox7.isSelected()?1:0,
                    jCheckBox8.isSelected()?1:0,
                    jCheckBox9.isSelected()?1:0,
                    jCheckBox10.isSelected()?1:0,
                    jCheckBox11.isSelected()?1:0,
                    jCheckBox12.isSelected()?1:0,
                    jCheckBox13.isSelected()?1:0,
                    jCheckBox14.isSelected()?1:0,
                    jCheckBox15.isSelected()?1:0,
                    jCheckBox16.isSelected()?1:0,
                    jCheckBox17.isSelected()?1:0,
                    jCheckBox18.isSelected()?1:0,
                    jCheckBox19.isSelected()?1:0,
                    jCheckBox20.isSelected()?1:0,
                    jCheckBox21.isSelected()?1:0,
                };
                reinit(step03_2(a), a);
            }
        });

        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setVisible(false);
                dialog4.setVisible(true);
            }
        });

        setLocation(getOwner().getWidth()/2-getWidth()/2, 0);
      }

      /** Fills the table with message. */
      public void reinit(Vector message, int[] features) {

          // get column headers
          columnId = new Vector();
          columnId.add("word");
          columnId.add("context");
          if (features[0]==1) {columnId.add(jCheckBox1.getText());}
          if (features[1]==1) {columnId.add(jCheckBox2.getText());}
          if (features[2]==1) {columnId.add(jCheckBox3.getText());}
          if (features[3]==1) {columnId.add(jCheckBox4.getText());}
          if (features[4]==1) {columnId.add(jCheckBox5.getText());}
          if (features[5]==1) {columnId.add(jCheckBox6.getText());}
          if (features[6]==1) {columnId.add(jCheckBox7.getText());}
          if (features[7]==1) {columnId.add(jCheckBox8.getText());}
          if (features[8]==1) {columnId.add(jCheckBox9.getText());}
          if (features[9]==1) {columnId.add(jCheckBox10.getText());}
          if (features[10]==1) {columnId.add(jCheckBox11.getText());}
          if (features[11]==1) {columnId.add(jCheckBox12.getText());}
          if (features[12]==1) {columnId.add(jCheckBox13.getText());}
          if (features[13]==1) {columnId.add(jCheckBox14.getText());}
          if (features[14]==1) {columnId.add(jCheckBox15.getText());}
          if (features[15]==1) {columnId.add(jCheckBox16.getText());}
          if (features[16]==1) {columnId.add(jCheckBox17.getText());}
          if (features[17]==1) {columnId.add(jCheckBox18.getText());}
          if (features[18]==1) {columnId.add(jCheckBox19.getText());}
          if (features[19]==1) {columnId.add(jCheckBox20.getText());}
          if (features[20]==1) {columnId.add(jCheckBox21.getText());}
          columnId.add("actual value (certain if it's an abbreviation)");
          columnId.add("predicted value (certain if it's an abbreviation)");
          columnId.add("process that determined the actual value");

          // set size, columns type and editability
          jTable1.setModel(new javax.swing.table.DefaultTableModel(
                                          message.size(), columnId.size()) {
                  public Class getColumnClass(int columnIndex) {
                      return String.class;
                  }
                  public  boolean isCellEditable(int row, int column) {
                      return false;
                  }
              });

          // set tooltip cell
          jTable1.setDefaultRenderer(String.class,
                   new javax.swing.table.DefaultTableCellRenderer() {
                   public java.awt.Component getTableCellRendererComponent (
                                   JTable table, Object value,
                                   boolean isSelected, boolean hasFocus,
                                   int row, int column) {

                       java.awt.Component cr =
                           super.getTableCellRendererComponent(table,
                                  value, isSelected, hasFocus, row, column);
                       setToolTipText((String)columnId.get(column)
                                      +": ["+value+"]");
                       return cr;
                  }
              });

          // set column headers
          ((javax.swing.table.DefaultTableModel)jTable1.getModel())
              .setColumnIdentifiers(columnId);

          // fills in the table cells
          Vector line = new Vector();
          for (int row = 0; row < message.size(); row++) {
              if (message.get(row) instanceof Vector) {
                  // more than one column
                  line = (Vector)message.get(row);
                  for (int column = 0; column < line.size(); column++) {
                      jTable1.setValueAt(line.get(column), row, column);
                  }

              } else {
                  // one multicolumn
                  // FIXME: text for this line expands on other columns
                  // impossible with JTable
                  jTable1.setValueAt((String)message.get(row), row, 0);
              }
          }
      }

    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBox2;
    private javax.swing.JCheckBox jCheckBox3;
    private javax.swing.JCheckBox jCheckBox4;
    private javax.swing.JCheckBox jCheckBox5;
    private javax.swing.JCheckBox jCheckBox6;
    private javax.swing.JCheckBox jCheckBox7;
    private javax.swing.JCheckBox jCheckBox8;
    private javax.swing.JCheckBox jCheckBox9;
    private javax.swing.JCheckBox jCheckBox10;
    private javax.swing.JCheckBox jCheckBox11;
    private javax.swing.JCheckBox jCheckBox12;
    private javax.swing.JCheckBox jCheckBox13;
    private javax.swing.JCheckBox jCheckBox14;
    private javax.swing.JCheckBox jCheckBox15;
    private javax.swing.JCheckBox jCheckBox16;
    private javax.swing.JCheckBox jCheckBox17;
    private javax.swing.JCheckBox jCheckBox18;
    private javax.swing.JCheckBox jCheckBox19;
    private javax.swing.JCheckBox jCheckBox20;
    private javax.swing.JCheckBox jCheckBox21;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTable jTable1;
  }

  /**
   * Show the user the unpredicted instances with several learning
   * algorithms. Allow the user to choose features used to learn.
   */
  public class LearnCapitalizedWordsDialog extends javax.swing.JDialog {
    
      Vector columnId;

      /** Creates new form dialog */
      public LearnCapitalizedWordsDialog(java.awt.Frame parent,
                                      boolean modal, Vector message,
                                      int[] features) {
          super(parent, modal);
          initComponents();
          reinit(message, features);

          jCheckBox1.setSelected((features[0]==1)?true:false);
          jCheckBox2.setSelected((features[1]==1)?true:false);
          jCheckBox3.setSelected((features[2]==1)?true:false);
          jCheckBox4.setSelected((features[3]==1)?true:false);
          jCheckBox5.setSelected((features[4]==1)?true:false);
          jCheckBox6.setSelected((features[5]==1)?true:false);
          jCheckBox7.setSelected((features[6]==1)?true:false);
          jCheckBox8.setSelected((features[7]==1)?true:false);
          jCheckBox9.setSelected((features[8]==1)?true:false);
          jCheckBox10.setSelected((features[9]==1)?true:false);
          jCheckBox11.setSelected((features[10]==1)?true:false);
          jCheckBox12.setSelected((features[11]==1)?true:false);
          jCheckBox13.setSelected((features[12]==1)?true:false);
          jCheckBox14.setSelected((features[13]==1)?true:false);
          jCheckBox15.setSelected((features[14]==1)?true:false);
          jCheckBox16.setSelected((features[15]==1)?true:false);
          jCheckBox17.setSelected((features[16]==1)?true:false);
          jCheckBox18.setSelected((features[17]==1)?true:false);
          jCheckBox19.setSelected((features[18]==1)?true:false);
          jCheckBox20.setSelected((features[19]==1)?true:false);
          jCheckBox21.setSelected((features[20]==1)?true:false);
          jCheckBox22.setSelected((features[21]==1)?true:false);

          getRootPane().setDefaultButton(jButton2);
          jButton2.requestFocus();
      }

      /** This method is called from within the constructor to
       * initialize the form.
       */
      private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jCheckBox2 = new javax.swing.JCheckBox();
        jCheckBox3 = new javax.swing.JCheckBox();
        jCheckBox4 = new javax.swing.JCheckBox();
        jCheckBox5 = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        jCheckBox6 = new javax.swing.JCheckBox();
        jCheckBox7 = new javax.swing.JCheckBox();
        jCheckBox8 = new javax.swing.JCheckBox();
        jLabel4 = new javax.swing.JLabel();
        jCheckBox9 = new javax.swing.JCheckBox();
        jCheckBox10 = new javax.swing.JCheckBox();
        jCheckBox11 = new javax.swing.JCheckBox();
        jCheckBox12 = new javax.swing.JCheckBox();
        jCheckBox13 = new javax.swing.JCheckBox();
        jLabel5 = new javax.swing.JLabel();
        jCheckBox14 = new javax.swing.JCheckBox();
        jCheckBox15 = new javax.swing.JCheckBox();
        jCheckBox16 = new javax.swing.JCheckBox();
        jCheckBox17 = new javax.swing.JCheckBox();
        jCheckBox18 = new javax.swing.JCheckBox();
        jCheckBox19 = new javax.swing.JCheckBox();
        jCheckBox20 = new javax.swing.JCheckBox();
        jCheckBox21 = new javax.swing.JCheckBox();
        jCheckBox22 = new javax.swing.JCheckBox();

        setTitle("Step 5/7 - Classify capitalized words");
        getContentPane().setLayout(new java.awt.BorderLayout(10, 10));
        setDefaultCloseOperation(javax.swing.WindowConstants
                                 .DISPOSE_ON_CLOSE);

        jLabel1.setText(
          "<html>Please choose the features you want to use for learning. "+
          "Add or remove a feature in ticking check boxes on the "+
          "right pane."+
          "<br>You can see the unpredicted instances in the table. "+
          "First with C4.5 learning algorithm then JRip on precedent "+
          "C4.5 unpredicted instances."+
          "<br>It should help you to determine which feature to keep, "+
          "remove or add."+
          "<br>Processes that determine the value of a word is, in that "+
          "order: cap. dict., anti-cap. dict., syntactic rules, "+
          "statistics, common words dict., learned model."+
          "<br>You can change the <b>class</b> feature of annotations "+
          "<b>CapitalizedWord</b> directly in the text but it has sense "+
          "only on a line with a possible actual value.");
        getContentPane().add(jLabel1, java.awt.BorderLayout.NORTH);

        jButton1.setText("Do it again");
        jPanel2.add(jButton1);
        jButton2.setText("Next Step");
        jPanel2.add(jButton2);
        getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

        jSplitPane1.setResizeWeight(1);
        jSplitPane1.setContinuousLayout(true);

        jScrollPane1.setAutoscrolls(true);
        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jTable1.setAutoResizeMode(javax.swing.JTable
                                  .AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        jTable1.setCellSelectionEnabled(true);
        jScrollPane1.setViewportView(jTable1);
        jSplitPane1.setLeftComponent(jScrollPane1);

        jPanel1.setLayout(new java.awt.GridLayout(26, 2, 5, 5));
        jPanel1.setBorder(new javax.swing.border.EtchedBorder());

        jLabel2.setText("Global features on possible capitalized word");
        jPanel1.add(jLabel2);

        jCheckBox1.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox1.setText("with upper initial in unambiguous position");
        jPanel1.add(jCheckBox1);

        jCheckBox2.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox2.setText("with upper initial in ambiguous position");
        jPanel1.add(jCheckBox2);

        jCheckBox3.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox3.setText("without upper initial");
        jPanel1.add(jCheckBox3);

        jCheckBox4.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox4.setText("different words before");
        jPanel1.add(jCheckBox4);

        jCheckBox5.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox5.setText("different words after");
        jPanel1.add(jCheckBox5);

        jCheckBox6.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox6.setText("capitalized expressions containing this word");
        jPanel1.add(jCheckBox6);

        jCheckBox7.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox7.setText("belongs to lexicon");
        jPanel1.add(jCheckBox7);

        jLabel3.setText("Local features on possible capitalized word");
        jPanel1.add(jLabel3);

        jCheckBox8.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox8.setText("length");
        jPanel1.add(jCheckBox8);

        jCheckBox9.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox9.setText("case");
        jPanel1.add(jCheckBox9);

        jCheckBox10.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox10.setText("contains digit");
        jPanel1.add(jCheckBox10);

        jLabel4.setText("Local features on previous word");
        jPanel1.add(jLabel4);

        jCheckBox11.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox11.setText("previous word grammatical category");
        if (!POSTagging) { jCheckBox11.setEnabled(false); }
        jPanel1.add(jCheckBox11);

        jCheckBox12.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox12.setText("previous word with upper initial");
        jPanel1.add(jCheckBox12);

        jCheckBox13.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox13.setText("previous word case");
        jPanel1.add(jCheckBox13);

        jCheckBox14.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox14.setText("previous word prefix");
        jPanel1.add(jCheckBox14);

        jCheckBox15.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox15.setText("previous word suffix");
        jPanel1.add(jCheckBox15);

        jLabel5.setText("Local features on next word");
        jPanel1.add(jLabel5);

        jCheckBox16.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox16.setText("next word grammatical category");
        if (!POSTagging) { jCheckBox16.setEnabled(false); }
        jPanel1.add(jCheckBox16);

        jCheckBox17.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox17.setText("next word with upper initial");
        jPanel1.add(jCheckBox17);

        jCheckBox18.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox18.setText("next word case");
        jPanel1.add(jCheckBox18);

        jCheckBox19.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox19.setText("next word prefix");
        jPanel1.add(jCheckBox19);

        jCheckBox20.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox20.setText("next word suffix");
        jPanel1.add(jCheckBox20);

        jCheckBox21.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox21.setText("user define feature 1");
        jPanel1.add(jCheckBox21);

        jCheckBox22.setFont(new java.awt.Font("Dialog", 0, 12));
        jCheckBox22.setText("user define feature 2");
        jPanel1.add(jCheckBox22);

        jSplitPane1.setRightComponent(jPanel1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        pack();

        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                int[] a = {
                    jCheckBox1.isSelected()?1:0,
                    jCheckBox2.isSelected()?1:0,
                    jCheckBox3.isSelected()?1:0,
                    jCheckBox4.isSelected()?1:0,
                    jCheckBox5.isSelected()?1:0,
                    jCheckBox6.isSelected()?1:0,
                    jCheckBox7.isSelected()?1:0,
                    jCheckBox8.isSelected()?1:0,
                    jCheckBox9.isSelected()?1:0,
                    jCheckBox10.isSelected()?1:0,
                    jCheckBox11.isSelected()?1:0,
                    jCheckBox12.isSelected()?1:0,
                    jCheckBox13.isSelected()?1:0,
                    jCheckBox14.isSelected()?1:0,
                    jCheckBox15.isSelected()?1:0,
                    jCheckBox16.isSelected()?1:0,
                    jCheckBox17.isSelected()?1:0,
                    jCheckBox18.isSelected()?1:0,
                    jCheckBox19.isSelected()?1:0,
                    jCheckBox20.isSelected()?1:0,
                    jCheckBox21.isSelected()?1:0,
                    jCheckBox22.isSelected()?1:0,
                };
                reinit(step05_2(a), a);
            }
        });

        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setVisible(false);
                dialog6.setVisible(true);
            }
        });

        setLocation(getOwner().getWidth()/2-getWidth()/2, 0);
      }

      /** Fills the table with message. */
      public void reinit(Vector message, int[] features) {

          // get column headers
          columnId = new Vector();
          columnId.add("word");
          columnId.add("context");
          if (features[0]==1) {columnId.add(jCheckBox1.getText());}
          if (features[1]==1) {columnId.add(jCheckBox2.getText());}
          if (features[2]==1) {columnId.add(jCheckBox3.getText());}
          if (features[3]==1) {columnId.add(jCheckBox4.getText());}
          if (features[4]==1) {columnId.add(jCheckBox5.getText());}
          if (features[5]==1) {columnId.add(jCheckBox6.getText());}
          if (features[6]==1) {columnId.add(jCheckBox7.getText());}
          if (features[7]==1) {columnId.add(jCheckBox8.getText());}
          if (features[8]==1) {columnId.add(jCheckBox9.getText());}
          if (features[9]==1) {columnId.add(jCheckBox10.getText());}
          if (features[10]==1) {columnId.add(jCheckBox11.getText());}
          if (features[11]==1) {columnId.add(jCheckBox12.getText());}
          if (features[12]==1) {columnId.add(jCheckBox13.getText());}
          if (features[13]==1) {columnId.add(jCheckBox14.getText());}
          if (features[14]==1) {columnId.add(jCheckBox15.getText());}
          if (features[15]==1) {columnId.add(jCheckBox16.getText());}
          if (features[16]==1) {columnId.add(jCheckBox17.getText());}
          if (features[17]==1) {columnId.add(jCheckBox18.getText());}
          if (features[18]==1) {columnId.add(jCheckBox19.getText());}
          if (features[19]==1) {columnId.add(jCheckBox20.getText());}
          if (features[20]==1) {columnId.add(jCheckBox21.getText());}
          if (features[21]==1) {columnId.add(jCheckBox22.getText());}
          columnId.add("actual value (certain if it's a capitalized word)");
          columnId
              .add("predicted value (certain if it's a capitalized word)");
          columnId.add("process that determined the actual value");

          // set size, columns type and editability
          jTable1.setModel(new javax.swing.table.DefaultTableModel(
                                          message.size(), columnId.size()) {
                  public Class getColumnClass(int columnIndex) {
                      return String.class;
                  }
                  public  boolean isCellEditable(int row, int column) {
                      return false;
                  }
              });

          // set tooltip cell
          jTable1.setDefaultRenderer(String.class,
                   new javax.swing.table.DefaultTableCellRenderer() {
                   public java.awt.Component getTableCellRendererComponent (
                                   JTable table, Object value,
                                   boolean isSelected, boolean hasFocus,
                                   int row, int column) {

                       java.awt.Component cr =
                           super.getTableCellRendererComponent(table,
                                  value, isSelected, hasFocus, row, column);
                       setToolTipText((String)columnId.get(column)
                                      +": ["+value+"]");
                       return cr;
                  }
              });

          // set column headers
          ((javax.swing.table.DefaultTableModel)jTable1.getModel())
              .setColumnIdentifiers(columnId);

          // fills in the table cells
          Vector line = new Vector();
          for (int row = 0; row < message.size(); row++) {
              if (message.get(row) instanceof Vector) {
                  // more than one column
                  line = (Vector)message.get(row);
                  for (int column = 0; column < line.size(); column++) {
                      jTable1.setValueAt(line.get(column), row, column);
                  }

              } else {
                  // one multicolumn
                  jTable1.setValueAt((String)message.get(row), row, 0);
              }
          }
      }

    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBox2;
    private javax.swing.JCheckBox jCheckBox3;
    private javax.swing.JCheckBox jCheckBox4;
    private javax.swing.JCheckBox jCheckBox5;
    private javax.swing.JCheckBox jCheckBox6;
    private javax.swing.JCheckBox jCheckBox7;
    private javax.swing.JCheckBox jCheckBox8;
    private javax.swing.JCheckBox jCheckBox9;
    private javax.swing.JCheckBox jCheckBox10;
    private javax.swing.JCheckBox jCheckBox11;
    private javax.swing.JCheckBox jCheckBox12;
    private javax.swing.JCheckBox jCheckBox13;
    private javax.swing.JCheckBox jCheckBox14;
    private javax.swing.JCheckBox jCheckBox15;
    private javax.swing.JCheckBox jCheckBox16;
    private javax.swing.JCheckBox jCheckBox17;
    private javax.swing.JCheckBox jCheckBox18;
    private javax.swing.JCheckBox jCheckBox19;
    private javax.swing.JCheckBox jCheckBox20;
    private javax.swing.JCheckBox jCheckBox21;
    private javax.swing.JCheckBox jCheckBox22;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTable jTable1;
  }

  /**
   * Determine CapitalizedWord words according to Abbreviations and
   * conversely.
   */
  public class Step06Dialog extends javax.swing.JDialog {
    
      /** Creates new form dialog */
      public Step06Dialog(java.awt.Frame parent, boolean modal,
                          int[] parameters) {

          super(parent, modal);
          initComponents();
          jSlider1.setValue(parameters[0]);
          jSlider2.setValue(parameters[1]);
          this.getRootPane().setDefaultButton(jButton3);
      }

      /** This method is called from within the constructor to
       * initialize the form.
       */
      private void initComponents() {
          jLabel4 = new javax.swing.JLabel();
          jPanel1 = new javax.swing.JPanel();
          jLabel1 = new javax.swing.JLabel();
          jSlider1 = new javax.swing.JSlider();
          jLabel2 = new javax.swing.JLabel();
          jSlider2 = new javax.swing.JSlider();
          jPanel2 = new javax.swing.JPanel();
          jButton1 = new javax.swing.JButton();
          jButton2 = new javax.swing.JButton();
          jButton3 = new javax.swing.JButton();

          getContentPane().setLayout(new java.awt.BorderLayout(20, 20));

          setDefaultCloseOperation(
              javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
          setTitle("Step 6/7 - Classify possible capitalized words and "+
                   "abbreviations");
          setBackground(new java.awt.Color(204, 204, 204));
          setLocationRelativeTo(null);

          jLabel4.setText(
            "<html>Please, choose the threshold to change CapWordPossible"+
            "<br>into CapWordImpossible and CapWordCertain according to"+
            "<br>the occurrence of AbbrImpossible and AbbrCertain that"+
            "<br>precede them. And conversely change AbbrPossible into"+
            "<br>AbbrCertain and AbbrImpossible."+
            "<br><br>Use the Replace function to change Annotations."+
            "</html>");
          getContentPane().add(jLabel4, java.awt.BorderLayout.NORTH);

          jPanel1.setLayout(new java.awt.GridLayout(4, 1, 10, 10));

          jLabel1.setText(
            "<html>Minimum number of total occurrences in statistics"+
            "<br>needed to use them."+
            "<br>Total occurrences = Certain count + Impossible count."+
            "</html>");
          jPanel1.add(jLabel1);

          jSlider1.setMinimum(0);
          jSlider1.setMaximum(50);
          jSlider1.setValue(2);
          jSlider1.setPaintLabels(true);
          jSlider1.setPaintTicks(true);
          jSlider1.setSnapToTicks(true);
          jSlider1.setMinorTickSpacing(1);
          jSlider1.setMajorTickSpacing(5);
          jPanel1.add(jSlider1);

          jLabel2.setText(
            "<html>Ratio between occurrences for Certain and Impossible"+
            "<br>capitalized words and abbreviations to class Possible"+
            "<br>abbreviations and capitalized words in certain or"+
            "<br>impossible classes</html>");
          jPanel1.add(jLabel2);

          jSlider2.setMinimum(1);
          jSlider2.setMaximum(10);
          jSlider2.setValue(3);
          jSlider2.setPaintLabels(true);
          jSlider2.setPaintTicks(true);
          jSlider2.setSnapToTicks(true);
          jSlider2.setMinorTickSpacing(1);
          jSlider2.setMajorTickSpacing(1);
          jPanel1.add(jSlider2);

          getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

          jPanel2.setLayout(new java.awt.FlowLayout(
                            java.awt.FlowLayout.CENTER, 10, 10));

          jButton1.setText("Process CapWordPossible");
          jButton1.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {

                      AnnotationSet AS = document.getAnnotations();
                      gate.AnnotationSet aAS;
                      gate.Annotation A;
                      Object[] array;

                      // delete New tags
                      HashSet AbbrAS = new HashSet();
                      AbbrAS.add("CapWordCertainNew");
                      AbbrAS.add("CapWordImpossibleNew");
                      aAS = AS.get(AbbrAS);
                      if (aAS != null) {
                      array = aAS.toArray();
                      for (int i = 0; i < array.length ; i++) {
                          A = (gate.Annotation)array[i];
                          AS.remove(A);
                      }
                      }

                      step06(new Double(jSlider1.getValue()).doubleValue(),
                             new Double(jSlider2.getValue()).doubleValue(),
                             true);
                  }
              });

          jPanel2.add(jButton1);

          jButton2.setText("Process AbbrPossible");
          jButton2.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {

                      AnnotationSet AS = document.getAnnotations();
                      gate.AnnotationSet aAS;
                      gate.Annotation A;
                      Object[] array;

                      // delete New tags
                      HashSet AbbrAS = new HashSet();
                      AbbrAS.add("AbbrCertainNew");
                      AbbrAS.add("AbbrImpossibleNew");
                      aAS = AS.get(AbbrAS);
                      if (aAS != null) {
                      array = aAS.toArray();
                      for (int i = 0; i < array.length ; i++) {
                          A = (gate.Annotation)array[i];
                          AS.remove(A);
                      }
                      }

                      step06(new Double(jSlider1.getValue()).doubleValue(),
                             new Double(jSlider2.getValue()).doubleValue(),
                             false);
                  }
              });

          jPanel2.add(jButton2);

          jButton3.setText("Next Step");
          jButton3.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent
                                              evt) {
                      statisticsValues[6] =
                          new Double(jSlider1.getValue()).doubleValue();
                      statisticsValues[7] =
                          new Double(jSlider2.getValue()).doubleValue();
                      setVisible(false);
                      dialog7.setVisible(true);
                  }
              });

          jPanel2.add(jButton3);

          getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

          pack();
          setLocation(getOwner().getWidth()/2-getWidth()/2, 0);
      }
   
      private javax.swing.JButton jButton1;
      private javax.swing.JButton jButton2;
      private javax.swing.JButton jButton3;
      private javax.swing.JLabel jLabel1;
      private javax.swing.JLabel jLabel2;
      private javax.swing.JPanel jPanel1;
      private javax.swing.JPanel jPanel2;
      private javax.swing.JSlider jSlider1;
      private javax.swing.JSlider jSlider2;
      private javax.swing.JLabel jLabel4;
  }

  /**
   * Notifies all the PRs in this controller that they should stop their
   * execution as soon as possible.
   */
  public synchronized void interrupt() {
      interrupted = true;
      gazetteer.interrupt();
      transducer_clean.interrupt();
      transducer02_2.interrupt();
      transducer03_1.interrupt();
      transducer04_1.interrupt();
      transducer05_1.interrupt();
      transducer07_3.interrupt();
  }

  public void setCorpusSampleDocument(gate.Document _corpusSampleDocument) {
      corpusSampleDocument = _corpusSampleDocument;
  }

  public gate.Document getCorpusSampleDocument() {
      return corpusSampleDocument;
  }

  public void setCorpus(gate.Corpus _corpus) {
      corpus = _corpus;
  }

  public gate.Corpus getCorpus() {
      return corpus;
  }

  public void setlineAnnotationsToSkipList(List
                                           newlineAnnotationsToSkipList) {
      lineAnnotationsToSkipList = newlineAnnotationsToSkipList;
  }

  public List getlineAnnotationsToSkipList() {
      return lineAnnotationsToSkipList;
  }

  public void setInputASName(String newInputASName) {
      inputASName = newInputASName;
  }

  public String getInputASName() {
      return inputASName;
  }

  public void setOutputASName(String newOutputASName) {
      outputASName = newOutputASName;
  }

  public String getOutputASName() {
      return outputASName;
  }

  public void setEncoding(String newEncoding) {
      encoding = newEncoding;
  }

  public String getEncoding() {
      return encoding;
  }

  public void setResourcesDirectoryURL(URL newResourcesDirectoryURL) {
      resourcesDirectoryURL = newResourcesDirectoryURL;
  }

  public URL getResourcesDirectoryURL() {
      return resourcesDirectoryURL;
  }

  public void setGazetteerFileURL(URL newGazetteerFileURL) {
      gazetteerFileURL = newGazetteerFileURL;
  }

  public URL getGazetteerFileURL() {
      return gazetteerFileURL;
  }

  public void setTransducerDirectoryURL(URL newTransducerDirectoryURL) {
      transducerDirectoryURL = newTransducerDirectoryURL;
  }

  public URL getTransducerDirectoryURL() {
      return transducerDirectoryURL;
  }

  public void setPrefixesFileURL(URL newPrefixesFileURL) {
      prefixesFileURL = newPrefixesFileURL;
  }

  public URL getPrefixesFileURL() {
      return prefixesFileURL;
  }

  public void setSuffixesFileURL(URL newSuffixesFileURL) {
      suffixesFileURL = newSuffixesFileURL;
  }

  public URL getSuffixesFileURL() {
      return suffixesFileURL;
  }

  public void setTagsFileURL(URL newTagsFileURL) {
      tagsFileURL = newTagsFileURL;
  }

  public URL getTagsFileURL() {
      return tagsFileURL;
  }

  public void setLexiconFileURL(URL newLexiconFileURL) {
      lexiconFileURL = newLexiconFileURL;
  }

  public URL getLexiconFileURL() {
      return lexiconFileURL;
  }

  public void setResultsDirectoryURL(URL newResultsDirectoryURL) {
      resultsDirectoryURL = newResultsDirectoryURL;
  }

  public URL getResultsDirectoryURL() {
      return resultsDirectoryURL;
  }

  public void setPOSTagging(Boolean newPOSTagging) {
      POSTagging = newPOSTagging.booleanValue();
  }

  public Boolean getPOSTagging() {
      return new Boolean(POSTagging);
  }

} // public class ActiveSegmentation
